@@ -75,6 +75,7 @@
 /*			FROM WWW
 **			--------
 */
+#include <LYUtils.h>
 #include <LYLeaks.h>
 
 #define DIRECTORY "/cnidr.org:210/directory-of-servers"
@@ -86,8 +87,6 @@
 
 #define HEX_ESCAPE '%'
 
-extern HTCJKlang HTCJK;
-
 extern int WWW_TraceFlag;	/* Control diagnostic output */
 extern FILE * logfile;		/* Log file output */
 
@@ -123,21 +122,22 @@ PRIVATE int fd_mosaic_connect_to_server ARGS3(
 	long,		port,
 	long *, 	fd)
 {
-    /*
-    **	New version.
-    */
-    char dummy[256];
+    char *dummy = NULL;
     int status;
+    int result;
 
-    sprintf (dummy, "wais://%s:%d/", host_name, port);
+    HTSprintf0(&dummy, "wais://%s:%d/", host_name, port);
 
     status = HTDoConnect (dummy, "WAIS", 210, (int *)fd);
     if (status == HT_INTERRUPTED) {
-	return -1;
+	result = -1;
+    } else if (status < 0) {
+	result = 0;
+    } else {
+	result = 1;
     }
-    if (status < 0)
-	return 0;
-    return 1;
+    FREE(dummy);
+    return result;
 }
 
 /* Returns 1 on success, 0 on fail, -1 on interrupt. */
@@ -161,16 +161,16 @@ PRIVATE int mosaic_connect_to_server ARGS3(
 
     rv = fd_mosaic_connect_to_server (host_name, port, &fd);
     if (rv == 0) {
-	HTAlert ("Could not connect to WAIS server.");
+	HTAlert (gettext("Could not connect to WAIS server."));
 	return 0;
     } else if (rv == -1) {
-	HTAlert ("Connection interrupted.");
+	HTAlert (CONNECTION_INTERRUPTED);
 	return -1;
     }
 
 #ifndef VMS
     if ((file = fdopen(fd,"r+")) == NULL) {
-	HTAlert ("Could not open WAIS connection for reading.");
+	HTAlert (gettext("Could not open WAIS connection for reading."));
 	return 0;
     }
 
@@ -194,11 +194,11 @@ PRIVATE void showDiags ARGS2(
 
     for (i = 0; d[i] != NULL; i++) {
 	if (d[i]->ADDINFO != NULL) {
-	    PUTS("Diagnostic code is ");
+	    PUTS(gettext("Diagnostic code is "));
 	    PUTS(d[i]->DIAG);
 	    PUTC(' ');
 	    PUTS(d[i]->ADDINFO);
-	    PUTC('\n'); ;
+	    PUTC('\n');
 	}
     }
 }
@@ -264,7 +264,9 @@ PRIVATE char * WWW_from_WAIS ARGS1(
     static char buf[BIG];
     char * q = buf;
     char * p = (docid->bytes);
+    char * result = NULL;
     int i, l;
+
     if (TRACE) {
 	char *p;
 	fprintf(tfp, "WAIS id (%d bytes) is ", (int)docid->size);
@@ -278,9 +280,9 @@ PRIVATE char * WWW_from_WAIS ARGS1(
     }
     for (p = docid->bytes;
 	 (p < docid->bytes+docid->size) && (q < &buf[BIG]);) {
-	CTRACE(tfp, "    Record type %d, length %d\n", p[0], p[1]);
+	CTRACE((tfp, "    Record type %d, length %d\n", p[0], p[1]));
 	if (*p > 10) {
-	    CTRACE(tfp, "Eh? DOCID record type of %d!\n", *p);
+	    CTRACE((tfp, "Eh?  DOCID record type of %d!\n", *p));
 	    return 0;
 	}
 	{	/* Bug fix -- allow any byte value 15 Apr 93 */
@@ -309,14 +311,9 @@ PRIVATE char * WWW_from_WAIS ARGS1(
 	*q++= ';';		/* Terminate field */
     }
     *q++ = 0;			/* Terminate string */
-    CTRACE(tfp, "WWW form of id: %s\n", buf);
-    {
-	char * result = (char *)malloc(strlen(buf)+1);
-	if (!result)
-	    outofmem(__FILE__, "WWW_from_WAIS");
-	strcpy(result, buf);
-	return result;
-    }
+    CTRACE((tfp, "WWW form of id: %s\n", buf));
+    StrAllocCopy(result, buf);
+    return result;
 } /* WWW_from_WAIS */
 
 /*	Transform URL into WAIS document identifier
@@ -340,7 +337,7 @@ PRIVATE any * WAIS_from_WWW ARGS2(
     char *s;	/* Position of semicolon */
     int n;	/* size */
 
-    CTRACE(tfp, "WWW id (to become WAIS id): %s\n", docname);
+    CTRACE((tfp, "WWW id (to become WAIS id): %s\n", docname));
     for (n = 0, p = docname; *p; p++) { /* Count sizes of strings */
 	n++;
 	if (*p == ';')
@@ -433,7 +430,7 @@ PRIVATE void output_text_record ARGS4(
 
   for (count = 0; count < record->DocumentText->size; count++){
     long ch = (unsigned char)record->DocumentText->bytes[count];
-    if (ch == 27) {	/* What is this in for? Tim */
+    if (ch == 27) {	/* What is this in for?  Tim */
 	    /* then we have an escape code */
 	    /* if the next letter is '(' or ')', then ignore two letters */
 	    if ('(' == record->DocumentText->bytes[count + 1] ||
@@ -465,12 +462,12 @@ PRIVATE void display_search_response ARGS4(
 
     BOOL archie =  strstr(database, "archie")!=0;	/* Specical handling */
 
-    CTRACE(tfp, "HTWAIS: Displaying search response\n");
-    PUTS("Index ");
+    CTRACE((tfp, "HTWAIS: Displaying search response\n"));
+    PUTS(gettext("Index "));
     START(HTML_EM);
     PUTS(database);
     END(HTML_EM);
-    sprintf(line, " contains the following %d item%s relevant to \"",
+    sprintf(line, gettext(" contains the following %d item%s relevant to \""),
 	    (int)(response->NumberOfRecordsReturned),
 	    response->NumberOfRecordsReturned ==1 ? "" : "s");
     PUTS(line);
@@ -478,11 +475,11 @@ PRIVATE void display_search_response ARGS4(
     PUTS(keywords);
     END(HTML_EM);
     PUTS("\".\n");
-    PUTS("The first figure after each entry is its relative score, ");
-    PUTS("the second is the number of lines in the item.");
+    PUTS(gettext("The first figure after each entry is its relative score, "));
+    PUTS(gettext("the second is the number of lines in the item."));
     START(HTML_BR);
     START(HTML_BR);
-    PUTS("\n");
+    PUTC('\n');
     START(HTML_OL);
 
     if (response->DatabaseDiagnosticRecords != 0) {
@@ -503,11 +500,11 @@ PRIVATE void display_search_response ARGS4(
 		/*
 		**  Make a printable string out of the document id.
 		*/
-		CTRACE(tfp, "HTWAIS:  %2ld: Score: %4ld, lines:%4ld '%s'\n",
+		CTRACE((tfp, "HTWAIS:  %2ld: Score: %4ld, lines:%4ld '%s'\n",
 			    i,
 			    (long int)(info->DocHeaders[k]->Score),
 			    (long int)(info->DocHeaders[k]->Lines),
-			    headline);
+			    headline));
 
 		START(HTML_LI);
 
@@ -520,29 +517,32 @@ PRIVATE void display_search_response ARGS4(
 			FREE(www_name);
 		    } else {
 			PUTS(headline);
-			PUTS(" (bad file name)");
+			PUTS(gettext(" (bad file name)"));
 		    }
 		} else { /* Not archie */
 		    docname =  WWW_from_WAIS(docid);
 		    if (docname) {
-			char * dbname = HTEscape(database, URL_XPALPHAS);
-			sprintf(line,
-				"/%s/%s/%d/%s", 	/* W3 address */
-				dbname,
-				head->Types ? head->Types[0] : "TEXT",
-				(int)(head->DocumentLength),
-				docname);
-			HTStartAnchor(target, NULL,
-				      ((head->Types) &&
-				       (!strcmp(head->Types[0], "URL")))
-						?
-				       headline : line); /* NT, Sep 93 */
+			if ((head->Types) &&
+			   (!strcmp(head->Types[0], "URL"))) {
+			    HTStartAnchor(target, NULL, headline);
+			} else{
+			    char * dbname = HTEscape(database, URL_XPALPHAS);
+			    char * w3_address = NULL;
+			    HTSprintf0(&w3_address,
+					"/%s/%s/%d/%s",
+					dbname,
+					head->Types ? head->Types[0] : "TEXT",
+					(int)(head->DocumentLength),
+					docname);
+			    HTStartAnchor(target, NULL, w3_address);
+			    FREE(w3_address);
+			    FREE(dbname);
+			}
 			PUTS(headline);
 			END(HTML_A);
-			FREE(dbname);
 			FREE(docname);
 		    } else {
-			PUTS("(bad doc id)");
+			PUTS(gettext("(bad doc id)"));
 		    }
 		}
 
@@ -558,21 +558,21 @@ PRIVATE void display_search_response ARGS4(
 	    k = 0;
 	    while (info->ShortHeaders[k] != 0) {
 		i++;
-		PUTS( "(Short Header record, can't display)");
+		PUTS( gettext("(Short Header record, can't display)"));
 	    }
 	}
 	if (info->LongHeaders != 0) {
 	    k = 0;
 	    while (info->LongHeaders[k] != 0) {
 		i++;
-		PUTS( "\nLong Header record, can't display\n");
+		PUTS( gettext("\nLong Header record, can't display\n"));
 	    }
 	}
 	if (info->Text != 0) {
 	    k = 0;
 	    while (info->Text[k] != 0) {
 		i++;
-		PUTS( "\nText record\n");
+		PUTS( gettext("\nText record\n"));
 		output_text_record((HTStream*)target,
 				   info->Text[k++], false, false);
 	    }
@@ -581,7 +581,7 @@ PRIVATE void display_search_response ARGS4(
 	    k = 0;
 	    while (info->Headlines[k] != 0) {
 		i++;
-		PUTS( "\nHeadline record, can't display\n");
+		PUTS( gettext("\nHeadline record, can't display\n"));
 		/* dsply_headline_record( info->Headlines[k++]); */
 	    }
 	}
@@ -589,13 +589,13 @@ PRIVATE void display_search_response ARGS4(
 	    k = 0;
 	    while (info->Codes[k] != 0) {
 		i++;
-		PUTS( "\nCode record, can't display\n");
+		PUTS( gettext("\nCode record, can't display\n"));
 		/* dsply_code_record( info->Codes[k++]); */
 	    }
 	}
     }	/* Loop: display user info */
     END(HTML_OL);
-    PUTC('\n'); ;
+    PUTC('\n');
 }
 
 /*		Load by name					HTLoadWAIS
@@ -663,7 +663,7 @@ PUBLIC int HTLoadWAIS ARGS4(
     }
     if (names[0] == '/') {
 	server_name = names+1;
-	if (as_gate =(*server_name == '/'))
+	if ((as_gate =(*server_name == '/')) != 0)
 	    server_name++;	/* Accept one or two */
 	www_database = strchr(server_name,'/');
 	if (www_database) {
@@ -693,9 +693,9 @@ PUBLIC int HTLoadWAIS ARGS4(
     }
 
     if (!ok)
-	return HTLoadError(sink, 500, "Syntax error in WAIS URL");
+	return HTLoadError(sink, 500, gettext("Syntax error in WAIS URL"));
 
-    CTRACE(tfp, "HTWAIS: Parsed OK\n");
+    CTRACE((tfp, "HTWAIS: Parsed OK\n"));
 
     service = strchr(names, ':');
     if (service)
@@ -712,16 +712,16 @@ PUBLIC int HTLoadWAIS ARGS4(
 
     } else if (!(key && !*key)) {
 	int status;
-	CTRACE (tfp, "===WAIS=== calling mosaic_connect_to_server\n");
+	CTRACE((tfp, "===WAIS=== calling mosaic_connect_to_server\n"));
 	status = mosaic_connect_to_server(server_name,
 					  atoi(service),
 					  &connection);
 	if (status == 0) {
-	    CTRACE (tfp, "===WAIS=== connection failed\n");
+	    CTRACE((tfp, "===WAIS=== connection failed\n"));
 	    FREE(names);
 	    return HT_NOT_LOADED;
 	} else if (status == -1) {
-	    CTRACE (tfp, "===WAIS=== connection interrupted\n");
+	    CTRACE((tfp, "===WAIS=== connection interrupted\n"));
 	    FREE(names);
 	    return HT_NOT_LOADED;
 	}
@@ -747,68 +747,69 @@ PUBLIC int HTLoadWAIS ARGS4(
 
     /*
     **	If keyword search is performed but there are no keywords,
-    **	the user has followed a link to the index itself. It would be
+    **	the user has followed a link to the index itself.  It would be
     **	appropriate at this point to send him the .SRC file - how?
     */
     if (key && !*key) { 			/* I N D E X */
 #ifdef CACHE_FILE_PREFIX
-	char filename[256];
+	char * filename = NULL;
 	FILE * fp;
 #endif
 	HTStructured * target = HTML_new(anAnchor, format_out, sink);
 
 	START(HTML_HEAD);
-	PUTS("\n");
-	HTStartIsIndex(target, "Enter WAIS query: ", NULL);
-	PUTS("\n");
+	PUTC('\n');
+	HTStartIsIndex(target, HTWAIS_SOLICIT_QUERY , NULL);
+	PUTC('\n');
 
 	{
 	    START(HTML_TITLE);
 	    PUTS(wais_database);
-	    PUTS(" (WAIS Index)");
+	    PUTS(gettext(" (WAIS Index)"));
 	    END(HTML_TITLE);
-	    PUTS("\n");
+	    PUTC('\n');
 	    END(HTML_HEAD);
-	    PUTS("\n");
+	    PUTC('\n');
 
 	    START(HTML_H1);
-	    PUTS("WAIS Index: ");
+	    PUTS(gettext("WAIS Index: "));
 	    START(HTML_EM);
 	    PUTS(wais_database);
 	    END(HTML_EM);
 	    END(HTML_H1);
-	    PUTS("\n");
-	    PUTS("This is a link for searching the ");
+	    PUTC('\n');
+	    PUTS(gettext("This is a link for searching the "));
 	    START(HTML_EM);
 	    PUTS(wais_database);
 	    END(HTML_EM);
-	    PUTS(" WAIS Index.\n");
+	    PUTS(gettext(" WAIS Index.\n"));
 
 	}
 	/*
 	**  If we have seen a source file for this database, use that.
 	*/
 #ifdef CACHE_FILE_PREFIX
-	sprintf(filename, "%sWSRC-%s:%s:%.100s.txt",
+	HTSprintf0(&filename, "%sWSRC-%s:%s:%.100s.txt",
 		CACHE_FILE_PREFIX,
 		server_name, service, www_database);
 
 	fp = fopen(filename, "r");	/* Have we found this already? */
-	CTRACE(tfp, "HTWAIS: Description of server %s %s.\n",
+	CTRACE((tfp, "HTWAIS: Description of server %s %s.\n",
 		    filename,
-		    fp ? "exists already" : "does NOT exist!");
+		    fp ? "exists already" : "does NOT exist!"));
 
 	if (fp) {
 	    char c;
 	    START(HTML_PRE);		/* Preformatted description */
-	    PUTS("\n");
+	    PUTC('\n');
 	    while((c=getc(fp))!=EOF) PUTC(c);	/* Transfer file */
 	    END(HTML_PRE);
 	    fclose(fp);
 	}
+	FREE(filename);
 #endif
 	START(HTML_P);
-	PUTS("\nEnter the 's'earch command and then specify search words.\n");
+	PUTS(gettext("\nEnter the 's'earch command and then specify search words.\n"));
 
 	FREE_TARGET;
     } else if (key) {					/* S E A R C H */
@@ -816,7 +817,8 @@ PUBLIC int HTLoadWAIS ARGS4(
 	HTStructured * target;
 
 	strncpy(keywords, key, MAX_KEYWORDS_LENGTH);
-	while(p=strchr(keywords, '+')) *p = ' ';
+	while ((p = strchr(keywords, '+')) != 0)
+	    *p = ' ';
 
 	/*
 	**  Send advance title to get something fast to the other end.
@@ -824,40 +826,40 @@ PUBLIC int HTLoadWAIS ARGS4(
 	target = HTML_new(anAnchor, format_out, sink);
 
 	START(HTML_HEAD);
-	PUTS("\n");
-	HTStartIsIndex(target, "Enter WAIS query: ", NULL);
-	PUTS("\n");
+	PUTC('\n');
+	HTStartIsIndex(target, HTWAIS_SOLICIT_QUERY, NULL);
+	PUTC('\n');
 	START(HTML_TITLE);
 	PUTS(keywords);
-	PUTS(" (in ");
+	PUTS(gettext(" (in "));
 	PUTS(wais_database);
-	PUTS(")");
+	PUTC(')');
 	END(HTML_TITLE);
-	PUTS("\n");
+	PUTC('\n');
 	END(HTML_HEAD);
-	PUTS("\n");
+	PUTC('\n');
 
 	START(HTML_H1);
-	PUTS("WAIS Search of \"");
+	PUTS(gettext("WAIS Search of \""));
 	START(HTML_EM);
 	PUTS(keywords);
 	END(HTML_EM);
-	PUTS("\" in: ");
+	PUTS(gettext("\" in: "));
 	START(HTML_EM);
 	PUTS(wais_database);
 	END(HTML_EM);
 	END(HTML_H1);
-	PUTS("\n");
+	PUTC('\n');
 
 	request_buffer_length = MAX_MESSAGE_LEN; /* Amount left */
-	CTRACE(tfp, "HTWAIS: Search for `%s' in `%s'\n",
-		    keywords, wais_database);
+	CTRACE((tfp, "HTWAIS: Search for `%s' in `%s'\n",
+		    keywords, wais_database));
 	if(NULL ==
 	generate_search_apdu(request_message + HEADER_LENGTH,
 				&request_buffer_length,
 				keywords, wais_database, NULL, MAXDOCS)) {
 #ifdef VMS
-	    HTAlert ("HTWAIS: Request too large.");
+	    HTAlert (gettext("HTWAIS: Request too large."));
 	    return_status = HT_NOT_LOADED;
 	    FREE_TARGET;
 	    goto CleanUp;
@@ -866,7 +868,7 @@ PUBLIC int HTLoadWAIS ARGS4(
 #endif /* VMS */
 	}
 
-	HTProgress("Searching WAIS database...");
+	HTProgress(gettext("Searching WAIS database..."));
 	rv = interpret_message (request_message,
 				MAX_MESSAGE_LEN - request_buffer_length,
 				response_message,
@@ -876,13 +878,13 @@ PUBLIC int HTLoadWAIS ARGS4(
 			       );
 
 	if (rv == HT_INTERRUPTED) {
-	    HTAlert ("Search interrupted.");
+	    HTAlert (gettext("Search interrupted."));
 	    return_status = HT_INTERRUPTED;
 	    FREE_TARGET;
 	    goto CleanUp;
 	} else if (!rv) {
 #ifdef VMS
-	    HTAlert ("HTWAIS: Return message too large.");
+	    HTAlert (HTWAIS_MESSAGE_TOO_BIG);
 	    return_status = HT_NOT_LOADED;
 	    FREE_TARGET;
 	    goto CleanUp;
@@ -909,8 +911,8 @@ PUBLIC int HTLoadWAIS ARGS4(
 	any   doc_chunk;
 	any * docid = &doc_chunk;
 
-	CTRACE(tfp, "HTWAIS: Retrieve document id `%s' type `%s' length %ld\n",
-		    docname, doctype, document_length);
+	CTRACE((tfp, "HTWAIS: Retrieve document id `%s' type `%s' length %ld\n",
+		    docname, doctype, document_length));
 
 	format_in =
 	  !strcmp(doctype, "WSRC") ? HTAtom_for("application/x-wais-source") :
@@ -926,7 +928,7 @@ PUBLIC int HTLoadWAIS ARGS4(
 	target = HTStreamStack(format_in, format_out, sink, anAnchor);
 	if (!target)
 	    return HTLoadError(sink, 500,
-			       "Can't convert format of WAIS document");
+			       gettext("Can't convert format of WAIS document"));
 	/*
 	**  Decode hex or litteral format for document ID.
 	*/
@@ -946,10 +948,10 @@ PUBLIC int HTLoadWAIS ARGS4(
 	    char *type = s_strdup(doctype);	/* Gets freed I guess */
 #endif /* VMS */
 	    request_buffer_length = MAX_MESSAGE_LEN; /* Amount left */
-	    CTRACE(tfp, "HTWAIS: Slice number %ld\n", count);
+	    CTRACE((tfp, "HTWAIS: Slice number %ld\n", count));
 
 	    if (HTCheckForInterrupt()) {
-		HTAlert ("Data transfer interrupted.");
+		HTAlert (TRANSFER_INTERRUPTED);
 		(*target->isa->_abort)(target, NULL);
 #ifdef VMS
 		FREE(type);
@@ -970,7 +972,7 @@ PUBLIC int HTLoadWAIS ARGS4(
 					type,
 					wais_database)) {
 #ifdef VMS
-		HTAlert ("HTWAIS: Request too long.");
+		HTAlert (gettext("HTWAIS: Request too long."));
 		return_status = HT_NOT_LOADED;
 		FREE_TARGET;
 		FREE(type);
@@ -984,7 +986,7 @@ PUBLIC int HTLoadWAIS ARGS4(
 	    /*
 	    **	Actually do the transaction given by request_message.
 	    */
-	    HTProgress("Fetching WAIS document...");
+	    HTProgress(gettext("Fetching WAIS document..."));
 	    rv = interpret_message(request_message,
 				   MAX_MESSAGE_LEN - request_buffer_length,
 				   response_message,
@@ -993,7 +995,7 @@ PUBLIC int HTLoadWAIS ARGS4(
 				   false /* true verbose */
 				  );
 	    if (rv == HT_INTERRUPTED) {
-		HTAlert ("Data transfer interrupted.");
+		HTAlert (TRANSFER_INTERRUPTED);
 		return_status = HT_INTERRUPTED;
 		FREE_TARGET;
 		FREE(type);
@@ -1001,7 +1003,7 @@ PUBLIC int HTLoadWAIS ARGS4(
 		goto CleanUp;
 	    } else if (!rv) {
 #ifdef VMS
-		HTAlert ("HTWAIS: Return message too large.");
+		HTAlert (HTWAIS_MESSAGE_TOO_BIG);
 		return_status = HT_NOT_LOADED;
 		FREE_TARGET;
 		FREE(type);
@@ -1023,7 +1025,7 @@ PUBLIC int HTLoadWAIS ARGS4(
 		 retrieval_response->DatabaseDiagnosticRecords)->Text) {
 		/* display_search_response(target, retrieval_response,
 					wais_database, keywords); */
-		PUTS("No text was returned!\n");
+		PUTS(gettext("No text was returned!\n"));
 		/* panic("No text was returned"); */
 	    } else {
 		output_text_record(target,
