@@ -4,16 +4,17 @@
 #include <LYStrings.h>
 #include <LYBookmark.h>
 #include <LYGlobalDefs.h>
-#include <LYSignal.h>
+#include <LYClean.h>
 #include <LYKeymap.h>
 #include <LYCharUtils.h> /* need for META charset */
+#include <UCAux.h>
 #include <LYCharSets.h>  /* need for LYHaveCJKCharacterSet */
 #include <LYCurses.h>
 #include <GridText.h>
+#include <HTCJK.h>
 
 #ifdef VMS
 #include <nam.h>
-extern BOOLEAN HadVMSInterrupt; /* Flag from cleanup_sig() AST */
 #endif /* VMS */
 
 #include <LYLeaks.h>
@@ -24,6 +25,18 @@ PUBLIC char *MBM_A_subdescript[MBM_V_MAXFILES+1];
 PRIVATE BOOLEAN is_mosaic_hotlist = FALSE;
 PRIVATE char * convert_mosaic_bookmark_file PARAMS((char *filename_buffer));
 
+PRIVATE void
+show_bookmark_not_defined NOARGS
+{
+    char *string_buffer = 0;
+
+    HTSprintf0(&string_buffer,
+	    BOOKMARK_FILE_NOT_DEFINED,
+	    key_for_func(LYK_OPTIONS));
+    LYMBM_statusline(string_buffer);
+    FREE(string_buffer);
+}
+
 /*
  *  Tries to open a bookmark file for reading, which may be
  *  the default, or based on offering the user a choice from
@@ -40,8 +53,8 @@ PRIVATE char * convert_mosaic_bookmark_file PARAMS((char *filename_buffer));
 PUBLIC char * get_bookmark_filename ARGS1(
 	char **,	URL)
 {
-    static char filename_buffer[256];
-    char string_buffer[256];
+    static char filename_buffer[LY_MAXPATH];
+    char *string_buffer = 0;
     FILE *fp;
     int MBM_tmp;
 
@@ -56,10 +69,7 @@ PUBLIC char * get_bookmark_filename ARGS1(
 	 */
 	return("");
     if (MBM_tmp == -1) {
-	sprintf(string_buffer,
-		BOOKMARK_FILE_NOT_DEFINED,
-		key_for_func(LYK_OPTIONS));
-	HTAlert(string_buffer);
+	show_bookmark_not_defined();
 	/*
 	 *  Space flags an undefined selection. - FMG
 	 */
@@ -79,13 +89,12 @@ PUBLIC char * get_bookmark_filename ARGS1(
     /*
      *	Seek it in the home path. - FM
      */
-    filename_buffer[255] = '\0';
     LYAddPathToHome(filename_buffer,
 		    sizeof(filename_buffer),
 		    BookmarkPage);
-    CTRACE(tfp, "\nget_bookmark_filename: SEEKING %s\n   AS %s\n\n",
-		BookmarkPage, filename_buffer);
-    if ((fp = fopen(filename_buffer,"r")) != NULL) {
+    CTRACE((tfp, "\nget_bookmark_filename: SEEKING %s\n   AS %s\n\n",
+		BookmarkPage, filename_buffer));
+    if ((fp = fopen(filename_buffer, TXT_R)) != NULL) {
 	goto success;
     }
 
@@ -99,21 +108,21 @@ success:
      *	We now have the file open.
      *	Check if it is a mosaic hotlist.
      */
-    if (fgets(string_buffer, 255, fp) &&
-	!strncmp(string_buffer, "ncsa-xmosaic-hotlist-format-1", 29)) {
+    if (LYSafeGets(&string_buffer, fp) != 0
+     && !strncmp(string_buffer, "ncsa-xmosaic-hotlist-format-1", 29)) {
 	char *newname;
 	/*
 	 *  It is a mosaic hotlist file.
 	 */
 	is_mosaic_hotlist = TRUE;
-	fclose(fp);
 	newname = convert_mosaic_bookmark_file(filename_buffer);
 	LYLocalFileToURL(URL, newname);
     } else {
-	fclose(fp);
 	is_mosaic_hotlist = FALSE;
 	LYLocalFileToURL(URL, filename_buffer);
     }
+    FREE(string_buffer);
+    fclose(fp);
 
     return(filename_buffer);  /* bookmark file exists */
 
@@ -124,36 +133,41 @@ success:
  *  file for handling as a Lynx bookmark file. - FM
  */
 PRIVATE char * convert_mosaic_bookmark_file ARGS1(
-	char *, 	filename_buffer)
+	char *,		filename_buffer)
 {
-    static char newfile[256];
+    static char newfile[LY_MAXPATH];
     FILE *fp, *nfp;
-    char buf[BUFSIZ];
+    char *buf = NULL;
     int line = -2;
     char *endline;
 
     LYRemoveTemp(newfile);
     if ((nfp = LYOpenTemp(newfile, HTML_SUFFIX, "w")) == NULL) {
 	LYMBM_statusline(NO_TEMP_FOR_HOTLIST);
-	sleep(AlertSecs);
+	LYSleepAlert();
 	return ("");
     }
 
-    if ((fp = fopen(filename_buffer, "r")) == NULL)
+    if ((fp = fopen(filename_buffer, TXT_R)) == NULL)
 	return ("");  /* should always open */
 
     fprintf(nfp,"<head>\n<title>%s</title>\n</head>\n",MOSAIC_BOOKMARK_TITLE);
-    fprintf(nfp,"\
+    fprintf(nfp, "%s\n\n<p>\n<ol>\n", gettext("\
      This file is an HTML representation of the X Mosaic hotlist file.\n\
      Outdated or invalid links may be removed by using the\n\
      remove bookmark command, it is usually the 'R' key but may have\n\
-     been remapped by you or your system administrator.\n\n<p>\n<ol>\n");
+     been remapped by you or your system administrator."));
 
-    while (fgets(buf, sizeof(buf), fp) != NULL) {
+    while ((LYSafeGets(&buf, fp)) != NULL) {
 	if(line >= 0) {
 	    endline = &buf[strlen(buf)-1];
 	    if(*endline == '\n')
 		*endline = '\0';
+#ifdef DOSPATH	/* 1998/01/10 (Sat) 15:41:35 */
+	    endline = strchr(buf, '\r');
+	    if (endline == NULL)
+		*endline = '\0';
+#endif
 	    if((line % 2) == 0) { /* even lines */
 		if(*buf != '\0') {
 		    strtok(buf," "); /* kill everything after the space */
@@ -171,8 +185,9 @@ PRIVATE char * convert_mosaic_bookmark_file ARGS1(
     return(newfile);
 }
 
-PRIVATE  BOOLEAN have8bit PARAMS((char *Title));
-PRIVATE  char* title_convert8bit PARAMS((char *Title));
+PRIVATE  BOOLEAN havevisible PARAMS((CONST char *Title));
+PRIVATE  BOOLEAN have8bit PARAMS((CONST char *Title));
+PRIVATE  char* title_convert8bit PARAMS((CONST char *Title));
 
 /*
  *  Adds a link to a bookmark file, creating the file
@@ -182,15 +197,16 @@ PRIVATE  char* title_convert8bit PARAMS((char *Title));
  *  to that file. - FM
  */
 PUBLIC void save_bookmark_link ARGS2(
-	char *, 	address,
-	char *, 	title)
+	char *,		address,
+	char *,		title)
 {
     FILE *fp;
     BOOLEAN first_time = FALSE;
     char *filename;
     char *bookmark_URL = NULL;
-    char filename_buffer[256];
-    char string_buffer[256];
+    char filename_buffer[LY_MAXPATH];
+    char string_buffer[BUFSIZ];
+    char tmp_buffer[BUFSIZ];
     char *Address = NULL;
     char *Title = NULL;
     int i, c;
@@ -228,7 +244,7 @@ PUBLIC void save_bookmark_link ARGS2(
 	    FREE(bookmark_URL);
 	    return;
 	}
-	strcpy(filename_buffer, filename);
+	LYstrncpy(filename_buffer, filename, sizeof(filename_buffer)-1);
     }
 
     /*
@@ -259,17 +275,32 @@ PUBLIC void save_bookmark_link ARGS2(
     /*
      *	Allow user to change the title. - FM
      */
-    string_buffer[255] = '\0';
-    LYstrncpy(string_buffer, title, 255);
-    convert_to_spaces(string_buffer, FALSE);
-    LYMBM_statusline(TITLE_PROMPT);
-    LYgetstr(string_buffer, VISIBLE, sizeof(string_buffer), NORECALL);
-    if (*string_buffer == '\0') {
-	LYMBM_statusline(CANCELLED);
-	sleep(MessageSecs);
-	FREE(bookmark_URL);
-	return;
-    }
+    do {
+	if (HTCJK == JAPANESE) {
+	    switch(kanji_code) {
+	    case EUC:
+		TO_EUC((CONST unsigned char *) title, (unsigned char *) tmp_buffer);
+		break;
+	    case SJIS:
+		TO_SJIS((CONST unsigned char *) title, (unsigned char *) tmp_buffer);
+		break;
+	    default:
+		break;
+	    }
+	    LYstrncpy(string_buffer, tmp_buffer, sizeof(string_buffer)-1);
+	} else {
+	    LYstrncpy(string_buffer, title, sizeof(string_buffer)-1);
+	}
+	convert_to_spaces(string_buffer, FALSE);
+	LYMBM_statusline(TITLE_PROMPT);
+	LYgetstr(string_buffer, VISIBLE, sizeof(string_buffer), NORECALL);
+	if (*string_buffer == '\0') {
+	    LYMBM_statusline(CANCELLED);
+	    LYSleepMsg();
+	    FREE(bookmark_URL);
+	    return;
+	}
+    } while(!havevisible(string_buffer));
 
     /*
      *	Create the Title with any left-angle-brackets
@@ -280,11 +311,14 @@ PUBLIC void save_bookmark_link ARGS2(
      *  from display character set which may need changing.
      *  Do NOT convert any 8-bit chars if we have CJK display. - LP
      */
-    StrAllocCopy(Title, string_buffer);
+    LYformTitle(&Title, string_buffer);
     LYEntify(&Title, TRUE);
     if (UCSaveBookmarksInUnicode &&
-		have8bit(Title) && (!LYHaveCJKCharacterSet))
-	StrAllocCopy(Title, title_convert8bit(Title));
+	have8bit(Title) && (!LYHaveCJKCharacterSet)) {
+	char *p = title_convert8bit(Title);
+	FREE(Title);
+	Title = p;
+    }
 
     /*
      *	Create the bookmark file, if it doesn't exist already,
@@ -299,11 +333,12 @@ PUBLIC void save_bookmark_link ARGS2(
 			sizeof(filename_buffer),
 			BookmarkPage);
     }
-    CTRACE(tfp, "\nsave_bookmark_link: SEEKING %s\n   AS %s\n\n",
-		BookmarkPage, filename_buffer);
+    CTRACE((tfp, "\nsave_bookmark_link: SEEKING %s\n   AS %s\n\n",
+		BookmarkPage, filename_buffer));
     if ((fp = fopen(filename_buffer, (first_time ? "w" : "a+"))) == NULL) {
 	LYMBM_statusline(BOOKMARK_OPEN_FAILED);
-	sleep(AlertSecs);
+	LYSleepAlert();
+	FREE(Title);
 	FREE(bookmark_URL);
 	return;
     }
@@ -320,21 +355,35 @@ PUBLIC void save_bookmark_link ARGS2(
      */
     if (first_time) {
 	fprintf(fp,"<head>\n");
+#if defined(SH_EX) && !defined(_WINDOWS)	/* 1997/12/11 (Thu) 19:13:40 */
+	if (HTCJK != JAPANESE)
+	    LYAddMETAcharsetToFD(fp, -1);
+	else
+	    fprintf(fp, "<META %s %s>\n",
+		    "http-equiv=\"content-type\"",
+		    "conetnt=\"text/html;charset=iso-2022-jp\""); 
+#else
 	LYAddMETAcharsetToFD(fp, -1);
-	fprintf(fp,"<title>%s</title>\n</head>\n",BOOKMARK_TITLE);
-	fprintf(fp,"\
+#endif	/* !_WINDOWS */
+	fprintf(fp,"<title>%s</title>\n</head>\n", BOOKMARK_TITLE);
+#ifdef _WINDOWS
+	fprintf(fp,
+	    gettext("     You can delete links by the 'R' key<br>\n<ol>\n"));
+#else
+	fprintf(fp, "%s<br>\n%s\n\n<!--\n%s\n-->\n\n<p>\n<ol>\n",
+		    gettext("\
      You can delete links using the remove bookmark command.  It is usually\n\
      the 'R' key but may have been remapped by you or your system\n\
-     administrator.<br>\n\
+     administrator."),
+		    gettext("\
      This file also may be edited with a standard text editor to delete\n\
-     outdated or invalid links, or to change their order.\n\n\
-<!--\n\
+     outdated or invalid links, or to change their order."),
+		    gettext("\
 Note: if you edit this file manually\n\
       you should not change the format within the lines\n\
       or add other HTML markup.\n\
-      Make sure any bookmark link saved as a single line\n\
--->\n\n\
-     <p>\n<ol>\n");
+      Make sure any bookmark link is saved as a single line."));
+#endif	/* _WINDOWS */
     }
 
     /*
@@ -386,7 +435,7 @@ Note: if you edit this file manually\n\
     FREE(Address);
     FREE(bookmark_URL);
     LYMBM_statusline(OPERATION_DONE);
-    sleep(MessageSecs);
+    LYSleepMsg();
 }
 
 /*
@@ -398,32 +447,37 @@ Note: if you edit this file manually\n\
  */
 PUBLIC void remove_bookmark_link ARGS2(
 	int,		cur,
-	char *, 	cur_bookmark_page)
+	char *,		cur_bookmark_page)
 {
     FILE *fp, *nfp;
-    char buf[BUFSIZ];
+    char *buf = NULL;
     int n;
 #ifdef VMS
     char filename_buffer[NAM$C_MAXRSS+12];
     char newfile[NAM$C_MAXRSS+12];
+#define keep_tempfile FALSE
 #else
-    char filename_buffer[256];
-    char newfile[256];
+    char filename_buffer[LY_MAXPATH];
+    char newfile[LY_MAXPATH];
+    BOOLEAN keep_tempfile = FALSE;
+#ifdef UNIX
     struct stat stat_buf;
     mode_t mode;
+    BOOLEAN regular = FALSE;
+#endif /* UNIX */
 #endif /* VMS */
-    char homepath[256];
+    char homepath[LY_MAXPATH];
 
-    CTRACE(tfp, "remove_bookmark_link: deleting link number: %d\n", cur);
+    CTRACE((tfp, "remove_bookmark_link: deleting link number: %d\n", cur));
 
     if (!cur_bookmark_page)
 	return;
     LYAddPathToHome(filename_buffer,
 		    sizeof(filename_buffer),
 		    cur_bookmark_page);
-    CTRACE(tfp, "\nremove_bookmark_link: SEEKING %s\n   AS %s\n\n",
-		cur_bookmark_page, filename_buffer);
-    if ((fp = fopen(filename_buffer, "r")) == NULL) {
+    CTRACE((tfp, "\nremove_bookmark_link: SEEKING %s\n   AS %s\n\n",
+		cur_bookmark_page, filename_buffer));
+    if ((fp = fopen(filename_buffer, TXT_R)) == NULL) {
 	HTAlert(BOOKMARK_OPEN_FAILED_FOR_DEL);
 	return;
     }
@@ -440,6 +494,7 @@ PUBLIC void remove_bookmark_link ARGS2(
      *	Explicitly preserve bookmark file mode on Unix. - DSL
      */
     if (stat(filename_buffer, &stat_buf) == 0) {
+	regular = (S_ISREG(stat_buf.st_mode) && stat_buf.st_nlink == 1);
 	mode = ((stat_buf.st_mode & 0777) | 0600); /* make it writable */
 	(void) chmod(newfile, mode);
 	if ((nfp = LYReopenTemp(newfile)) == NULL) {
@@ -453,7 +508,7 @@ PUBLIC void remove_bookmark_link ARGS2(
     if (is_mosaic_hotlist) {
 	int del_line = cur*2;  /* two lines per entry */
 	n = -3;  /* skip past cookie and name lines */
-	while (fgets(buf, sizeof(buf), fp) != NULL) {
+	while (LYSafeGets(&buf, fp) != NULL) {
 	    n++;
 	    if (n == del_line || n == del_line+1)
 		continue;  /* remove two lines */
@@ -462,15 +517,19 @@ PUBLIC void remove_bookmark_link ARGS2(
 	}
 
     } else {
-	char *cp;
+	char *cp, *cp2;
 	BOOLEAN retain;
 	int seen;
 
 	n = -1;
-	while (fgets(buf, sizeof(buf), fp) != NULL) {
+	while (LYSafeGets(&buf, fp) != NULL) {
+	    int keep_ol = FALSE;
 	    retain = TRUE;
 	    seen = 0;
 	    cp = buf;
+	    if ((cur == 0) && (cp2 = LYstrstr(cp,"<ol><LI>")))
+		keep_ol = TRUE; /* Do not erase, this corrects a bug in an
+				   older version */
 	    while (n < cur && (cp = LYstrstr(cp, "<a href="))) {
 		seen++;
 		if (++n == cur) {
@@ -479,7 +538,9 @@ PUBLIC void remove_bookmark_link ARGS2(
 			HTAlert(BOOKMARK_LINK_NOT_ONE_LINE);
 			goto failure;
 		    }
-		    CTRACE(tfp, "remove_bookmark_link: skipping link %d\n", n);
+		    CTRACE((tfp, "remove_bookmark_link: skipping link %d\n", n));
+		    if (keep_ol)
+			fprintf(nfp,"<ol>\n");
 		    retain = FALSE;
 		}
 		cp += 8;
@@ -489,11 +550,16 @@ PUBLIC void remove_bookmark_link ARGS2(
 	}
     }
 
-    CTRACE(tfp, "remove_bookmark_link: files: %s %s\n",
-			newfile, filename_buffer);
+    FREE(buf);
+    CTRACE((tfp, "remove_bookmark_link: files: %s %s\n",
+			newfile, filename_buffer));
 
     fclose(fp);
     fp = NULL;
+    if (fflush(nfp) == EOF) {
+	CTRACE((tfp, "fflush(nfp): %s", LYStrerror(errno)));
+	goto failure;
+    }
     LYCloseTempFP(nfp);
     nfp = NULL;
 #ifdef DOSPATH
@@ -506,22 +572,39 @@ PUBLIC void remove_bookmark_link ARGS2(
      *	can preserve the original ownership of the file, provided that
      *	it is writable by the current process.
      *	Changed to copy  1998-04-26 -- gil
+     *  But if the copy fails, for example because the filesystem is full,
+     *  we are left with a corrupt bookmark file.  Changed back to use
+     *  the previous mechanism [try rename(), then mv for EXDEV], except
+     *  in usual cases (not a regular file e.g., symbolic link, or has hard
+     *  links).  This will let bookmarks survive a filesystem full condition
+     *  in the "normal" case (bookmark is on same filesystem as home directory,
+     *  is a regular file, has no additional hard links).
+     *  If we first tried LYCopyFile, and that fails, also fall back to trying
+     *  the other stuff.  That gives a chance to recover in case the LYCopyFile
+     *  left a corrupt target file.
+     *  If there is an error, and that error may mean that the bookmark file
+     *  has been corrupted, don't remove the temporary newfile (which should
+     *  always be uncorrupted) in place, it may still be used to recover
+     *  manually.  If this applies, produce an additional message to that
+     *  effect.  The temp file will still be removed by normal program exit
+     *  cleanup. - kw 1999-11-12
      */
-    {	char buffer[3072];
-
-	sprintf(buffer, "%s %s %s && %s %s",
-	    COPY_PATH, newfile, filename_buffer,
-	    RM_PATH, newfile);
-
-	CTRACE(tfp, "remove_bookmark_link: %s\n", buffer);
-	if( LYSystem( buffer ) ) {
-	    HTAlert(BOOKTEMP_COPY_FAIL);
-	} else {
+    if (!regular) {
+	if (LYCopyFile(newfile, filename_buffer) == 0) {
+	    LYRemoveTemp(newfile);
 	    return;
 	}
+	LYSleepAlert();	/* give a chance to see error from cp - kw */
+	HTUserMsg(BOOKTEMP_COPY_FAIL);
+	keep_tempfile = TRUE;
     }
-#else  /* !UNIX */
+#endif  /* UNIX */
+
     if (rename(newfile, filename_buffer) != -1) {
+#ifdef UNIX
+	if (regular)
+	    chmod(filename_buffer, stat_buf.st_mode & 07777);
+#endif /* UNIX */
 	HTSYS_purge(filename_buffer);
 	return;
     } else {
@@ -531,17 +614,41 @@ PUBLIC void remove_bookmark_link ARGS2(
 	 *  Check if this is the case and do something appropriate.
 	 *  Used to be ODD_RENAME
 	 */
-#ifdef _WINDOWS
+#if defined(_WINDOWS) || defined(WIN_EX)
+#if defined(WIN_EX)
+	if (GetLastError() == ERROR_NOT_SAME_DEVICE)
+#else /* !_WIN_EX */
 	if (errno == ENOTSAM)
-#else
-	if (errno == EXDEV)
-#endif /* WINDOWS */
+#endif /* _WIN_EX */
 	{
-	    char buffer[2048];
-	    sprintf(buffer, "%s %s %s", MV_PATH, newfile, filename_buffer);
-	    LYSystem(buffer);
-	    return;
+	    if (rename(newfile, filename_buffer) != 0) {
+		if (LYCopyFile(newfile, filename_buffer) == 0)
+		    remove(newfile);
+	    }
 	}
+#else
+	if (errno == EXDEV) {
+	    static CONST char MV_FMT[] = "%s %s %s";
+	    char *buffer = 0;
+	    HTAddParam(&buffer, MV_FMT, 1, MV_PATH);
+	    HTAddParam(&buffer, MV_FMT, 2, newfile);
+	    HTAddParam(&buffer, MV_FMT, 3, filename_buffer);
+	    HTEndParam(&buffer, MV_FMT, 3);
+	    if (LYSystem(buffer) == 0) {
+#ifdef UNIX
+		if (regular)
+		    chmod(filename_buffer, stat_buf.st_mode & 07777);
+#endif /* UNIX */
+		FREE(buffer);
+		return;
+	    } else {
+		FREE(buffer);
+		keep_tempfile = TRUE;
+		goto failure;
+	    }
+	}
+	CTRACE((tfp, "rename(): %s", LYStrerror(errno)));
+#endif /* _WINDOWS */
 #endif /* !VMS */
 
 #ifdef VMS
@@ -552,14 +659,21 @@ PUBLIC void remove_bookmark_link ARGS2(
 	if (TRACE)
 	    perror("renaming the file");
     }
-#endif /* UNIX */
+
 
 failure:
+    FREE(buf);
     HTAlert(BOOKMARK_DEL_FAILED);
-    LYCloseTempFP(nfp);
+    if (nfp)
+	LYCloseTempFP(nfp);
     if (fp != NULL)
 	fclose(fp);
-    LYRemoveTemp(newfile);
+    if (keep_tempfile) {
+	HTUserMsg2(gettext("File may be recoverable from %s during this session"),
+		   newfile);
+    } else {
+	LYRemoveTemp(newfile);
+    }
 }
 
 /*
@@ -651,8 +765,6 @@ PUBLIC int select_menu_multi_bookmarks NOARGS
 {
     int c, MBM_tmp_count, MBM_allow;
     int MBM_screens, MBM_from, MBM_to, MBM_current;
-    char string_buffer[256];
-    char shead_buffer[256];
 
     /*
      *	If not enabled, pick the "default" (0).
@@ -695,12 +807,6 @@ PUBLIC int select_menu_multi_bookmarks NOARGS
 	HTAlert(MULTIBOOKMARKS_SMALL);
 	return (-2);
     }
-    /*
-     *	Load the bad choice message buffer.
-     */
-    sprintf(string_buffer,
-	    BOOKMARK_FILE_NOT_DEFINED,
-	    key_for_func(LYK_OPTIONS));
 
     MBM_screens = (MBM_V_MAXFILES/MBM_allow)+1; /* int rounds off low. */
 
@@ -726,14 +832,16 @@ draw_bookmark_choices:
     move(1, 5);
     lynx_start_h1_color ();
     if (MBM_screens > 1) {
-	sprintf(shead_buffer,
+	char *shead_buffer = 0;
+	HTSprintf0(&shead_buffer,
 		MULTIBOOKMARKS_SHEAD_MASK, MBM_current, MBM_screens);
 	addstr(shead_buffer);
+	FREE(shead_buffer);
     } else {
 	addstr(MULTIBOOKMARKS_SHEAD);
     }
 
-   lynx_stop_h1_color ();
+    lynx_stop_h1_color ();
 
     MBM_tmp_count = 0;
     for (c = MBM_from; c <= MBM_to; c++) {
@@ -830,8 +938,7 @@ get_bookmark_choice:
     if (c < 0 || c > MBM_V_MAXFILES) {
 	goto get_bookmark_choice;
     } else if (!MBM_A_subbookmark[c]) {
-	LYMBM_statusline(string_buffer);
-	sleep(AlertSecs);
+	show_bookmark_not_defined();
 	LYMBM_statusline(MULTIBOOKMARKS_SAVE);
 	goto get_bookmark_choice;
     } else {
@@ -865,7 +972,7 @@ PUBLIC BOOLEAN LYHaveSubBookmarks NOARGS
  *  string will be handled properly. - FM
  */
 PUBLIC void LYMBM_statusline  ARGS1(
-	char *, 	text)
+	char *,		text)
 {
     if (LYMultiBookmarks == TRUE && user_mode == NOVICE_MODE) {
 	LYStatusLine = (LYlines - 1);
@@ -876,10 +983,39 @@ PUBLIC void LYMBM_statusline  ARGS1(
     }
 }
 
+/*
+ * Check whether we have any visible (non-blank) chars.
+ */
+PRIVATE  BOOLEAN havevisible ARGS1(CONST char *, Title)
+{
+    CONST char *p = Title;
+    unsigned char c;
+    long unicode;
+
+    for ( ; *p; p++) {
+	c = (unsigned char)(TOASCII(*p));
+	if (c > 32 && c < 127)
+	    return(TRUE);
+	if (c <= 32 || c == 127)
+	    continue;
+	if (LYHaveCJKCharacterSet || !UCCanUniTranslateFrom(current_char_set))
+	    return(TRUE);
+	unicode = UCTransToUni(*p, current_char_set);
+	if (unicode > 32 && unicode < 127)
+	    return(TRUE);
+	if (unicode <= 32 || unicode == 0xa0 || unicode == 0xad)
+	    continue;
+	if (unicode >= 0x2000 && unicode < 0x200f)
+	    continue;
+	return(TRUE);
+    }
+    return(FALSE); /* if we came here */
+}
+
 /*
  * Check whether string have 8 bit chars.
  */
-PRIVATE  BOOLEAN have8bit ARGS1(char *, Title)
+PRIVATE  BOOLEAN have8bit ARGS1(CONST char *, Title)
 {
     CONST char *p = Title;
 
@@ -892,7 +1028,7 @@ PRIVATE  BOOLEAN have8bit ARGS1(char *, Title)
 
 /*
  *  Ok, title have 8-bit characters and they are in display charset.
- *  Bookmarks is a permanent file. To avoid dependencies from display
+ *  Bookmarks is a permanent file.  To avoid dependencies from display
  *  character set which may be changed with time
  *  we store 8-bit characters as numeric character reference (NCR),
  *  so where the character encoded as unicode number in form of &#xUUUU;
@@ -909,42 +1045,53 @@ PRIVATE  BOOLEAN have8bit ARGS1(char *, Title)
  *  Older versions fail.
  *
  */
-PRIVATE  char* title_convert8bit ARGS1(char *, Title)
+PRIVATE  char* title_convert8bit ARGS1(CONST char *, Title)
 {
     CONST char *p = Title;
-    char temp[256];
-    char *q = temp;
+    char *p0;
+    char *q;
     char *comment = NULL;
     char *ncr     = NULL;
     char *buf = NULL;
+    int charset_in  = current_char_set;
+    int charset_out = UCGetLYhndl_byMIME("us-ascii");
 
     for ( ; *p; p++) {
-	LYstrncpy(q, p, 1);
-	if ((unsigned char)*q <= 127) {
-	    StrAllocCat(comment, q);
-	    StrAllocCat(ncr, q);
+	char temp[2];
+	LYstrncpy(temp, p, sizeof(temp)-1);
+	if ((unsigned char)*temp <= 127) {
+	    StrAllocCat(comment, temp);
+	    StrAllocCat(ncr, temp);
 	} else {
-	int charset_in, charset_out, uck;
-	long unicode;
-	char replace_buf [10], replace_buf2 [10];
+	    long unicode;
+	    char replace_buf [32];
 
-	charset_in  = current_char_set;
-	charset_out = UCGetLYhndl_byMIME("us-ascii");
+	    if (UCTransCharStr(replace_buf, sizeof(replace_buf), *temp,
+				  charset_in, charset_out, YES) > 0)
+		StrAllocCat(comment, replace_buf);
 
-	uck = UCTransCharStr(replace_buf, sizeof(replace_buf), *q,
-			      charset_in, charset_out, YES);
-	if (uck >0)
-	StrAllocCat(comment, replace_buf);
+	    unicode = UCTransToUni( *temp, charset_in);
 
-	unicode = UCTransToUni( *q, charset_in);
-
-	StrAllocCat(ncr, "&#");
-	sprintf(replace_buf2, "%ld", unicode);
-	StrAllocCat(ncr, replace_buf2);
-	StrAllocCat(ncr, ";");
+	    StrAllocCat(ncr, "&#");
+	    sprintf(replace_buf, "%ld", unicode);
+	    StrAllocCat(ncr, replace_buf);
+	    StrAllocCat(ncr, ";");
 	}
     }
 
+    /*
+     *  Cleanup comment, collapse multiple dashes into one dash,
+     *  skip '>'.
+     */
+    for (q = p0 = comment; *p0; p0++) {
+	if ((unsigned char)(TOASCII(*p0)) >= 32 &&
+	    *p0 != '>' &&
+	    (q == comment || *p0 != '-' || *(q-1) != '-')) {
+	    *q++ = *p0;
+	}
+    }
+    *q = '\0';
+
     /*
      * valid bookmark should be a single line (no linebreaks!).
      */
@@ -955,6 +1102,5 @@ PRIVATE  char* title_convert8bit ARGS1(char *, Title)
 
     FREE(comment);
     FREE(ncr);
-
     return(buf);
 }
