@@ -6,23 +6,21 @@
 #include <UCMap.h>
 #include <UCDefs.h>
 #include <LYCharSets.h>
-#include <LYCharUtils.h>
-#include <HTFont.h>
 #include <GridText.h>
 #include <LYCurses.h>
 #include <LYStrings.h>
 
-#include <LYexit.h>
 #include <LYLeaks.h>
 
 extern BOOL HTPassEightBitRaw;
 extern BOOL HTPassEightBitNum;
 extern BOOL HTPassHighCtrlRaw;
 extern BOOL HTPassHighCtrlNum;
-extern HTCJKlang HTCJK;
 PUBLIC HTkcode kanji_code = NOKANJI;
 PUBLIC BOOLEAN LYHaveCJKCharacterSet = FALSE;
 PUBLIC BOOLEAN DisplayCharsetMatchLocale = TRUE;
+PUBLIC BOOL force_old_UCLYhndl_on_reload = FALSE;
+PUBLIC int forced_UCLYhdnl;
 extern void UCInit NOARGS;
 extern int UCInitialized;
 PUBLIC int LYNumCharsets = 0;  /* Will be initialized later by UC_Register. */
@@ -30,19 +28,35 @@ PUBLIC int current_char_set = -1; /* will be intitialized later in LYMain.c */
 PUBLIC CONST char** p_entity_values = NULL; /* Pointer, for HTML_put_entity()*/
 			      /* obsolete and probably not used(???)        */
 			      /* will be initialized in HTMLUseCharacterSet */
+#ifdef EXP_CHARSET_CHOICE
+PUBLIC charset_subset_t charset_subsets[MAXCHARSETS];
+PUBLIC BOOL custom_display_charset = FALSE;
+PUBLIC BOOL custom_assumed_doc_charset = FALSE;
+#ifndef ALL_CHARSETS_IN_O_MENU_SCREEN
+PUBLIC int display_charset_map[MAXCHARSETS];
+PUBLIC int assumed_doc_charset_map[MAXCHARSETS];
+
+PUBLIC CONST char* display_charset_choices[MAXCHARSETS+1];
+PUBLIC CONST char* assumed_charset_choices[MAXCHARSETS+1];
+PUBLIC int displayed_display_charset_idx;
+#endif
+#endif /* EXP_CHARSET_CHOICE */
 
 /*
+ *  New character sets now declared with UCInit() in UCdomap.c
+ *
  *  INSTRUCTIONS for adding new character sets which do not have
- *		 Unicode tables.
+ *		 Unicode tables now in UCdomap.h
  *
- *  Currently we only declare some charset's properties here
- *  (such as MIME names, etc.), it does not include real mapping.
  *
- *  [We hope you need not correct/add old-style mapping
+ *  [We hope you need not correct/add old-style mapping below
  *  as in ISO_LATIN1[] or SevenBitApproximations[] any more -
  *  it works now via new chartrans mechanism, but kept for compatibility only:
  *  we should cleanup the stuff, but this is not so easy...]
  *
+ *  Currently we only declare some charset's properties here
+ *  (such as MIME names, etc.), it does not include real mapping.
+ *
  *  There is a place marked "Add your new character sets HERE" in this file.
  *  Make up a character set and add it in the same
  *  style as the ISO_LATIN1 set below, giving it a unique name.
@@ -77,7 +91,7 @@ PRIVATE CONST char * ISO_Latin1[] = {
 	"\313", /* capital E, dieresis or umlaut mark (&#203;) - Euml */
 	"\315", /* capital I, acute accent (&#205;) - Iacute */
 	"\316", /* capital I, circumflex accent (&#206;) - Icirc */
-	"\314", /* capital I, grave accent (&#200;) - Igrave */
+	"\314", /* capital I, grave accent (&#204;) - Igrave */
 	"\317", /* capital I, dieresis or umlaut mark (&#207;) - Iuml */
 	"\321", /* capital N, tilde (&#209;) - Ntilde */
 	"\323", /* capital O, acute accent (&#211;) - Oacute */
@@ -328,42 +342,16 @@ PUBLIC CONST char * SevenBitApproximations[] = {
  */
 PUBLIC CONST char ** LYCharSets[MAXCHARSETS]={
 	ISO_Latin1,		/* ISO Latin 1		*/
-	SevenBitApproximations, /* DosLatin1 (cp850)	*/
-	SevenBitApproximations, /* WinLatin1 (cp1252)	*/
-	SevenBitApproximations, /* DosLatinUS (cp437)	*/
-	SevenBitApproximations, /* DEC Multinational	*/
-	SevenBitApproximations, /* Macintosh (8 bit)	*/
-	SevenBitApproximations, /* NeXT character set	*/
-	SevenBitApproximations, /* Chinese		*/
-	SevenBitApproximations, /* Japanese (EUC-JP)	*/
-	SevenBitApproximations, /* Japanese (Shift_JIS)	*/
-	SevenBitApproximations, /* Korean		*/
-	SevenBitApproximations, /* Taipei (Big5)	*/
-	SevenBitApproximations, /* Vietnamese (VISCII)	*/
 	SevenBitApproximations, /* 7 Bit Approximations */
-	SevenBitApproximations, /* Transparent		*/
 };
 
 /*
  *  Add the name that the user will see below.
  *  The order of LYCharSets and LYchar_set_names MUST be the same
  */
-PUBLIC CONST char * LYchar_set_names[MAXCHARSETSP]={
+PUBLIC CONST char * LYchar_set_names[MAXCHARSETS + 1]={
 	"Western (ISO-8859-1)",
-	"Western (cp850)",
-	"Western (windows-1252)",
-	"IBM PC US codepage (cp437)",
-	"DEC Multinational",
-	"Macintosh (8 bit)",
-	"NeXT character set",
-	"Chinese",
-	"Japanese (EUC-JP)",
-	"Japanese (Shift_JIS)",
-	"Korean",
-	"Taipei (Big5)",
-	"Vietnamese (VISCII)",
 	"7 bit approximations (US-ASCII)",
-	"Transparent",
 	(char *) 0
 };
 
@@ -373,59 +361,29 @@ PUBLIC CONST char * LYchar_set_names[MAXCHARSETSP]={
  *  Will be automatically modified (and extended) by charset translations
  *  which are loaded using the chartrans mechanism.
  *  Most important piece of info to put here is a MIME charset name.
- *  Used for chartrans.
+ *  Used for chartrans (see UCDefs.h).
  *  The order of LYCharSets and LYCharSet_UC MUST be the same.
  *
  *  Note that most of the charsets added by the new mechanism in src/chrtrans
- *  don't show up here at all.	They don't have to.
+ *  don't show up here at all.  They don't have to.
  */
 PUBLIC LYUCcharset LYCharSet_UC[MAXCHARSETS]=
 {
   /*
    *  Zero position placeholder and HTMLGetEntityUCValue() reference. - FM
    */
-  {-1,"iso-8859-1",	UCT_ENC_8BIT,
-			UCT_REP_IS_LAT1,
-			UCT_CP_IS_LAT1, 	UCT_R_LAT1,UCT_R_LAT1},
+  {-1,"iso-8859-1",    UCT_ENC_8BIT,0,
+                       UCT_REP_IS_LAT1,
+                       UCT_CP_IS_LAT1,         UCT_R_LAT1,UCT_R_LAT1},
 
   /*
    *  Placeholders for Unicode tables. - FM
    */
-  {-1,"cp850",		UCT_ENC_8BIT,
-			UCT_REP_SUPERSETOF_LAT1,
-			0,			UCT_R_8BIT,UCT_R_ASCII},
-  {-1,"windows-1251", 	UCT_ENC_8BIT,
-			UCT_REP_SUPERSETOF_LAT1,
-			0,			UCT_R_8BIT,UCT_R_ASCII},
-  {-1,"cp437",		UCT_ENC_8BIT,0,0,	UCT_R_8BIT,UCT_R_ASCII},
-  {-1,"dec-mcs",	UCT_ENC_8BIT,0,0,	UCT_R_8BIT,UCT_R_ASCII},
-  {-1,"macintosh",	UCT_ENC_8BIT,0,0,	UCT_R_8BIT,UCT_R_ASCII},
-  {-1,"next",		UCT_ENC_8BIT,0,0,	UCT_R_8BIT,UCT_R_ASCII},
+  {-1,"us-ascii",      UCT_ENC_7BIT,0,
+                       UCT_REP_SUBSETOF_LAT1,
+                       UCT_CP_SUBSETOF_LAT1,   UCT_R_ASCII,UCT_R_ASCII},
 
-  /*
-   *  There is no strict correlation for the next five, since the transfer
-   *  charset gets decoded into Display Char Set by the CJK code (separate
-   *  from Unicode mechanism).	For now we use the MIME name that describes
-   *  what is output to the terminal. - KW
-   */
-  {-1,"euc-cn", 	UCT_ENC_CJK,0,0,	UCT_R_8BIT,UCT_R_ASCII},
-  {-1,"euc-jp", 	UCT_ENC_CJK,0,0,	UCT_R_8BIT,UCT_R_ASCII},
-  {-1,"shift_jis",	UCT_ENC_CJK,0,0,	UCT_R_8BIT,UCT_R_ASCII},
-  {-1,"euc-kr", 	UCT_ENC_CJK,0,0,	UCT_R_8BIT,UCT_R_ASCII},
-  {-1,"big5",		UCT_ENC_CJK,0,0,	UCT_R_8BIT,UCT_R_ASCII},
-
-  /*
-   *  Placeholders for Unicode tables. - FM
-   */
-  {-1,"viscii", 	UCT_ENC_8BIT_C0,0,0,	UCT_R_8BIT,UCT_R_ASCII},
-  {-1,"us-ascii",	UCT_ENC_7BIT,
-			UCT_REP_SUBSETOF_LAT1,
-			UCT_CP_SUBSETOF_LAT1,	UCT_R_ASCII,UCT_R_ASCII},
-
-  /*
-   *  Placeholder for non-translation mode. - FM
-   */
-  {-1,"x-transparent",	UCT_ENC_8BIT,0,0,	UCT_R_8BIT,UCT_R_ASCII}
+/*  {-1,"iso-8859-15",   UCT_ENC_8BIT,0,0,0,     UCT_R_8BIT,UCT_R_ASCII}, */
 
 };
 
@@ -439,20 +397,7 @@ PUBLIC LYUCcharset LYCharSet_UC[MAXCHARSETS]=
  */
 PUBLIC int LYlowest_eightbit[MAXCHARSETS]={
 	160,	/* ISO Latin 1		*/
-	128,	/* DosLatin1 (cp850)	*/
-	130,	/* WinLatin1 (cp1252)	*/
-	128,	/* DosLatinUS (cp437)	*/
-	160,	/* DEC Multinational	*/
-	128,	/* Macintosh (8 bit)	*/
-	128,	/* NeXT character set	*/
-	128,	/* Chinese		*/
-	128,	/* Japanese (EUC)	*/
-	128,	/* Japanese (SJIS)	*/
-	128,	/* Korean		*/
-	128,	/* Taipei (Big5)	*/
-	128,	/* Vietnamese (VISCII)	*/
 	999,	/* 7 bit approximations */
-	128	/* Transparent	(???)	*/
 };
 
 
@@ -463,6 +408,8 @@ PUBLIC int LYlowest_eightbit[MAXCHARSETS]={
 PUBLIC void HTMLSetCharacterHandling ARGS1(int,i)
 {
     int chndl = safeUCGetLYhndl_byMIME(UCAssume_MIMEcharset);
+    BOOLEAN LYRawMode_flag = LYRawMode;
+    int UCLYhndl_for_unspec_flag = UCLYhndl_for_unspec;
 
     if (LYCharSet_UC[i].enc != UCT_ENC_CJK) {
 	HTCJK = NOCJK;
@@ -470,19 +417,19 @@ PUBLIC void HTMLSetCharacterHandling ARGS1(int,i)
 	if (i == chndl)
 	    LYRawMode = LYUseDefaultRawMode;
 	else
-	    LYRawMode = (!LYUseDefaultRawMode);
+	    LYRawMode = (BOOL) (!LYUseDefaultRawMode);
 
-	HTPassEightBitNum =
+	HTPassEightBitNum = (BOOL) (
 	    ((LYCharSet_UC[i].codepoints & UCT_CP_SUPERSETOF_LAT1) ||
-		(LYCharSet_UC[i].like8859 & UCT_R_HIGH8BIT));
+		(LYCharSet_UC[i].like8859 & UCT_R_HIGH8BIT)));
 
 	if (LYRawMode) {
-	    HTPassEightBitRaw = (LYlowest_eightbit[i] <= 160);
+	    HTPassEightBitRaw = (BOOL) (LYlowest_eightbit[i] <= 160);
 	} else {
 	    HTPassEightBitRaw = FALSE;
 	}
 	if (LYRawMode || i == chndl) {
-	    HTPassHighCtrlRaw = (LYlowest_eightbit[i] <= 130);
+	    HTPassHighCtrlRaw = (BOOL) (LYlowest_eightbit[i] <= 130);
 	} else {
 	    HTPassHighCtrlRaw = FALSE;
 	}
@@ -512,10 +459,10 @@ PUBLIC void HTMLSetCharacterHandling ARGS1(int,i)
 	/* for any CJK: */
 	if (!LYUseDefaultRawMode)
 		HTCJK = NOCJK;
-	LYRawMode = (HTCJK != NOCJK) ? TRUE : FALSE;
+	LYRawMode = (BOOL) ((HTCJK != NOCJK) ? TRUE : FALSE);
 	HTPassEightBitRaw = FALSE;
 	HTPassEightBitNum = FALSE;
-	HTPassHighCtrlRaw = (HTCJK != NOCJK) ? TRUE : FALSE;
+	HTPassHighCtrlRaw = (BOOL) ((HTCJK != NOCJK) ? TRUE : FALSE);
 	HTPassHighCtrlNum = FALSE;
     }
 
@@ -548,7 +495,19 @@ PUBLIC void HTMLSetCharacterHandling ARGS1(int,i)
     }
 #endif /* USE_SLANG */
 
-    ena_csi((LYlowest_eightbit[current_char_set] > 155));
+    ena_csi((BOOLEAN)(LYlowest_eightbit[current_char_set] > 155));
+
+    /* some diagnostics */
+    if (TRACE) {
+	if (LYRawMode_flag != LYRawMode)
+	    CTRACE((tfp, "HTMLSetCharacterHandling: LYRawMode changed %s -> %s\n",
+			(LYRawMode_flag ? "ON" : "OFF"),
+			(LYRawMode	? "ON" : "OFF")));
+	if (UCLYhndl_for_unspec_flag != UCLYhndl_for_unspec)
+	    CTRACE((tfp, "HTMLSetCharacterHandling: UCLYhndl_for_unspec changed %d -> %d\n",
+			UCLYhndl_for_unspec_flag,
+			UCLYhndl_for_unspec));
+    }
 
     return;
 }
@@ -560,6 +519,8 @@ PUBLIC void Set_HTCJK ARGS2(
 	CONST char *,	inMIMEname,
 	CONST char *,	outMIMEname)
 {
+    /* need not check for synonyms: MIMEname's got from LYCharSet_UC */
+
     if (LYRawMode) {
 	if ((!strcmp(inMIMEname, "euc-jp") ||
 	     !strcmp(inMIMEname, "shift_jis")) &&
@@ -592,7 +553,7 @@ PUBLIC void Set_HTCJK ARGS2(
  */
 PRIVATE void HTMLSetRawModeDefault ARGS1(int,i)
 {
-    LYDefaultRawMode = (LYCharSet_UC[i].enc == UCT_ENC_CJK);
+    LYDefaultRawMode = (BOOL) (LYCharSet_UC[i].enc == UCT_ENC_CJK);
     return;
 }
 
@@ -611,7 +572,7 @@ PUBLIC void HTMLSetUseDefaultRawMode ARGS2(
 	if (i == chndl)
 	    LYUseDefaultRawMode = modeflag;
 	else
-	    LYUseDefaultRawMode = (!modeflag);
+	    LYUseDefaultRawMode = (BOOL) (!modeflag);
     } else /* CJK encoding: */
 	    LYUseDefaultRawMode = modeflag;
 
@@ -624,7 +585,7 @@ PUBLIC void HTMLSetUseDefaultRawMode ARGS2(
  */
 PRIVATE void HTMLSetHaveCJKCharacterSet ARGS1(int,i)
 {
-    LYHaveCJKCharacterSet = (LYCharSet_UC[i].enc == UCT_ENC_CJK);
+    LYHaveCJKCharacterSet = (BOOL) (LYCharSet_UC[i].enc == UCT_ENC_CJK);
     return;
 }
 
@@ -658,7 +619,12 @@ PRIVATE void HTMLSetDisplayCharsetMatchLocale ARGS1(int,i)
 	match = TRUE; /* guess, but see below */
 
 #if !defined(LOCALE)
-	match = FALSE;
+	if (LYCharSet_UC[i].enc != UCT_ENC_UTF8)
+	    /*
+	     *  Leave true for utf-8 display - the code doesn't deal
+	     *  very well with this case. - kw
+	     */
+	    match = FALSE;
 #else
 	if (UCForce8bitTOUPPER) {
 	    /*
@@ -879,7 +845,7 @@ PUBLIC CONST char * HTMLGetEntityName ARGS1(
 	UCode_t,	code)
 {
 #define IntValue code
-    int MaxValue = ((sizeof(LYEntityNames)/sizeof(char **)) - 1);
+    int MaxValue = (TABLESIZE(LYEntityNames) - 1);
 
     if (IntValue < 0 || IntValue > MaxValue) {
 	return "";
@@ -889,9 +855,8 @@ PUBLIC CONST char * HTMLGetEntityName ARGS1(
 }
 
 /*
- *  Function to return the UCode_t (long int) value for entity names
- *  in the ISO_Latin1 and UC_entity_info unicode_entities arrays.
- *  It returns 0 if not found. - FM
+ *  Function to return the UCode_t (long int) value for entity names.
+ *  It returns 0 if not found.
  *
  *  unicode_entities[] handles all the names from old style entities[] too.
  *  Lynx now calls unicode_entities[] only through this function:
@@ -905,10 +870,12 @@ PUBLIC CONST char * HTMLGetEntityName ARGS1(
 PUBLIC UCode_t HTMLGetEntityUCValue ARGS1(
 	CONST char *,	name)
 {
+#include <entities.h>
+
     UCode_t value = 0;
     size_t i, high, low;
     int diff = 0;
-    CONST UC_entity_info * unicode_entities = HTML_dtd.unicode_entity_info;
+    size_t number_of_unicode_entities = TABLESIZE(unicode_entities);
 
     /*
      *	Make sure we have a non-zero length name. - FM
@@ -919,20 +886,19 @@ PUBLIC UCode_t HTMLGetEntityUCValue ARGS1(
     /*
      *	Try UC_entity_info unicode_entities[].
      */
-    for (low = 0, high = HTML_dtd.number_of_unicode_entities;
+    for (low = 0, high = number_of_unicode_entities;
 	 high > low;
 	 diff < 0 ? (low = i+1) : (high = i)) {
 	/*
 	**  Binary search.
 	*/
 	i = (low + (high-low)/2);
-	diff = strcmp(unicode_entities[i].name, name);	/* Case sensitive! */
+	diff = AS_cmp(unicode_entities[i].name, name);	/* Case sensitive! */
 	if (diff == 0) {
 	    value = unicode_entities[i].code;
 	    break;
 	}
     }
-
     return(value);
 }
 
@@ -954,9 +920,43 @@ PUBLIC void HTMLUseCharacterSet ARGS1(int, i)
  *  Initializer, calls initialization function for the
  *  CHARTRANS handling. - KW
  */
-PUBLIC int LYCharSetsDeclared NOPARAMS
+PUBLIC int LYCharSetsDeclared NOARGS
 {
     UCInit();
 
     return UCInitialized;
 }
+
+#ifdef EXP_CHARSET_CHOICE
+PUBLIC void init_charset_subsets NOARGS
+{
+    int i,n;
+    int cur_display = 0;
+    int cur_assumed = 0;
+
+    /* add them to displayed values */
+    charset_subsets[UCLYhndl_for_unspec].hide_assumed = FALSE;
+    charset_subsets[current_char_set].hide_display = FALSE;
+
+#ifndef ALL_CHARSETS_IN_O_MENU_SCREEN
+    /*all this stuff is for supporting old menu screen... */
+    for (i = 0; i < LYNumCharsets; ++i){
+	if (charset_subsets[i].hide_display == FALSE) {
+	    n = cur_display++;
+	    if (i == current_char_set)
+		displayed_display_charset_idx = n;
+	    display_charset_map[n] = i;
+	    display_charset_choices[n] = LYchar_set_names[i];
+	}
+	if (charset_subsets[i].hide_assumed == FALSE) {
+	    n = cur_assumed++;
+	    assumed_doc_charset_map[n] = i;
+	    assumed_charset_choices[n] = LYCharSet_UC[i].MIMEname;
+	    charset_subsets[i].assumed_idx = n;
+	}
+	display_charset_choices[cur_display] = NULL;
+	assumed_charset_choices[cur_assumed] = NULL;
+    }
+#endif
+}
+#endif /* EXP_CHARSET_CHOICE */
