@@ -7,9 +7,14 @@
 
 /*******************************************************************
  * There are four sections to this document:
- *  Section 1.  Things you MUST change or verify
+ *  Section 1.  Things you MUST verify.  Unix platforms use a configure
+ *		script to provide sensible default values.  If your site
+ *		has special requirements, that may not be sufficient.
+ *		For non-Unix platforms (e.g., VMS), there is no
+ *		configure script, so the defaults here are more
+ *		critical.
  *	Section 1a)  VMS specific things
- *	Section 1b)  UNIX specific things
+ *	Section 1b)  non-VMS specific things
  *	Section 1c)  ALL Platforms
  *
  *  Section 2.  Things you should probably check!
@@ -35,7 +40,7 @@
  */
 
 /*******************************************************************
- * Things you must change  VMS specific
+ * Things you must change  -  VMS specific
  *  Section 1a).
  */
 #ifdef VMS
@@ -92,10 +97,11 @@
 #endif /* LYNX_CFG_FILE */
 
 /**************************
- * The EXTENSION_MAP file allows you to map file suffix's to
+ * The EXTENSION_MAP file allows you to map file suffixes to
  * mime types.
- * These global and personal files override anything in
- * lynx.cfg or src/HTInit.c
+ * The file locations defined here can be overridden in lynx.cfg.
+ * Mappings in these global and personal files override any SUFFIX
+ * definitions in lynx.cfg and built-in defaults from src/HTInit.c.
  */
 #define GLOBAL_EXTENSION_MAP "Lynx_Dir:mime.types"
 #define PERSONAL_EXTENSION_MAP "mime.types"
@@ -103,8 +109,9 @@
 /**************************
  * The MAILCAP file allows you to map file MIME types to
  * external viewers.
- * These global and personal files override anything in
- * lynx.cfg or src/HTInit.c
+ * The file locations defined here can be overridden in lynx.cfg.
+ * Mappings in these global and personal files override any VIEWER
+ * definitions in lynx.cfg and built-in defaults from src/HTInit.c.
  */
 #define GLOBAL_MAILCAP "Lynx_Dir:mailcap"
 #define PERSONAL_MAILCAP ".mailcap"
@@ -113,14 +120,15 @@
  * XLOADIMAGE_COMMAND will be used as a default in src/HTInit.c
  * for viewing image content types when the DECW$DISPLAY logical
  * is set.  Make it the foreign command for your system's X image
- * viewer (commonly, "xv").  Make it "exit" or something like that
- * if you don't have one.  It can be anything that will handle GIF,
+ * viewer (commonly, "xv").  It can be anything that will handle GIF,
  * TIFF and other popular image formats.  Freeware ports of xv for
  * VMS are available in the ftp://ftp.wku.edu/vms/unsupported and
  * http://www.openvms.digital.com/cd/XV310A/ subdirectories.  You
  * must also have a "%s" for the filename.  The default defined
  * here can be overridden in lynx.cfg, or via the global or personal
  * mailcap files.
+ * Make this NULL if you don't have such a viewer or don't want to
+ * use any default viewers for image types.
  */
 #define XLOADIMAGE_COMMAND "xv %s"
 
@@ -224,10 +232,10 @@
 #endif /* LYNX_LSS_FILE */
 
 /*******************************************************************
- * Things you must change  UNIX specific
+ * Things you must change  -  non-VMS specific
  *  Section 1b).
  */
-#else     /* UNIX */
+#else     /* non-VMS: UNIX etc. */
 
 /**************************
  * NOTE: This variable is set by the configure script; editing changes will
@@ -275,10 +283,11 @@
 #endif /* HAVE_CONFIG_H */
 
 /**************************
- * The EXTENSION_MAP file allows you to map file suffix's to
+ * The EXTENSION_MAP file allows you to map file suffixes to
  * mime types.
- * These global and personal files override anything in
- * lynx.cfg or src/HTInit.c
+ * The file locations defined here can be overridden in lynx.cfg.
+ * Mappings in these global and personal files override any SUFFIX
+ * definitions in lynx.cfg and built-in defaults from src/HTInit.c.
  */
 #define GLOBAL_EXTENSION_MAP "/etc/mime.types"
 #define PERSONAL_EXTENSION_MAP ".mime.types"
@@ -286,43 +295,29 @@
 /**************************
  * The MAILCAP file allows you to map file MIME types to
  * external viewers.
- * These global and personal files override anything in
- * lynx.cfg or src/HTInit.c
+ * The file locations defined here can be overridden in lynx.cfg.
+ * Mappings in these global and personal files override any VIEWER
+ * definitions in lynx.cfg and built-in defaults from src/HTInit.c.
  */
 #define GLOBAL_MAILCAP "/etc/mailcap"
 #define PERSONAL_MAILCAP ".mailcap"
 
-/**************************
- * the full path and name of the telnet command
- */
-#define TELNET_COMMAND "telnet"
-
-/**************************
- * the full path and name of the tn3270 command
- */
-#define TN3270_COMMAND "tn3270"
-
-/**************************
- * the full path and name of the rlogin command
- */
-#define RLOGIN_COMMAND "rlogin"
-
 /**************************
  * XLOADIMAGE_COMMAND will be used as a default in src/HTInit.c for
  * viewing image content types when the DISPLAY environment variable
  * is set.  Make it the full path and name of the xli (also known as
- * xloadimage or xview) command, or other image viewer.  Put 'echo' or
- * something like it here if you don't have a suitable viewer.  It can
- * be anything that will handle GIF, TIFF and other popular image formats
+ * xloadimage or xview) command, or other image viewer.  It can be
+ * anything that will handle GIF, TIFF and other popular image formats
  * (xli does).  The freeware distribution of xli is available in the
  * ftp://ftp.x.org/contrib/ subdirectory.  The shareware, xv, also is
  * suitable.  You must also have a "%s" for the filename; "&" for
  * background is optional.  The default defined here can be overridden
- * in lynx.cfg, or via the global or personal mailcap files.  Note that
- * open is used as the default for NeXT, instead of the XLOADIMAGE_COMMAND
- * definition.
+ * in lynx.cfg, or via the global or personal mailcap files.
+ * Make this NULL if you don't have such a viewer or don't want to
+ * use any default viewers for image types.  Note that open is used as
+ * the default for NeXT, instead of the XLOADIMAGE_COMMAND definition.
  */
-#define XLOADIMAGE_COMMAND "xli %s &"
+#define XLOADIMAGE_COMMAND NULL
 
 /**************************
  * For UNIX systems, SYSTEM_MAIL and SYSTEM_MAIL_FLAGS are set by the
@@ -330,7 +325,7 @@
  */
 
 /**************************
- * A place to put temporary files, it's almost always in "/tmp/"
+ * A place to put temporary files, it is almost always in "/tmp/"
  * for UNIX systems.  If you include "$USER" in the definition
  * (e.g., "/tmp/$USER"), Lynx will replace the "$USER" with the
  * username of the account which invoked the Lynx image.  Such
@@ -339,10 +334,29 @@
  * If the path includes a tilde (e.g, "~" or "~/lynxtmp"), Lynx will
  * replace the tilde with the full path for the user's home.
  * The definition here can be overridden at run time by setting a
- * "LYNX_TEMP_SPACE" environment symbol.
+ * "LYNX_TEMP_SPACE" environment variable, or (if that is not set)
+ * the "TMPDIR" (unix), or "TEMP" or "TMP" (Windows,DOS,OS/2)
+ * variable.
  */
 #define TEMP_SPACE "/tmp/"
 
+/********************************
+ * Comment this line out to disable code that randomizes the names given to
+ * temporary files.
+ */
+#define EXP_RAND_TEMPNAME 1
+
+/********************************
+ * Uncomment this line to use 'mkstemp()' in preference to lynx's own code
+ * in fmt_tempname().  Caution: on a few older systems, mkstemp() is less
+ * secure than you would like.  For this reason, we do not auto-configure it.
+ *
+ * Some things to watch out for:  some broken implementations of mkstemp() may
+ * not necessarily try a different filename when they are called, making it
+ * impossible to rename the file.  Others make world-writable files.
+ */
+/*#define USE_MKSTEMP */
+
 /********************************
  * Comment this line out to let the user enter his/her email address
  * when sending a message.  There should be no need to do this unless
@@ -351,7 +365,7 @@
  * upgrade, because anonymous mail makes it far too easy for a user to
  * spoof someone else's email address.)
  */
-#define NO_ANONYMOUS_EMAIL TRUE
+/*#define NO_ANONYMOUS_EMAIL TRUE */
 
 /********************************
  * LIST_FORMAT defines the display for local files when LONG_LIST
@@ -367,6 +381,8 @@
  *	%d	date of last modification
  *	%a	anchor pointing to file or directory
  *	%A	as above but don't show symbolic links
+ *	%t	type of file (description derived from MIME type)
+ *	%T	MIME type as known by Lynx (from mime.types or default)
  *	%k	size of file in Kilobytes
  *	%K	as above but omit size for directories
  *	%s	size of file in bytes
@@ -381,7 +397,11 @@
  *
  * For the Unix "ls -l" format:    "    %p %4l %-8.8o %-8.8g %7s %-12.12d %a"
  */
+#ifdef DOSPATH
+#define LIST_FORMAT "    %4K %-12.12d %a"
+#else
 #define LIST_FORMAT "    %p %4l %-8.8o %-8.8g %7s %-12.12d %a"
+#endif
 
 /*
  *  If NO_FORCED_CORE_DUMP is set to TRUE, Lynx will not force
@@ -419,13 +439,27 @@
  */
 
 /*****************************
- * STARTFILE is the default file if none is specified in lynx.cfg,
- *  on the command line, or via a WWW_HOME environment variable.
+ * STARTFILE is the default starting URL if none is specified
+ *   on the command line or via a WWW_HOME environment variable;
+ *   Lynx will refuse to start without a starting URL of some kind.
+ * STARTFILE can be remote, e.g., http://www.w3.org/default.html ,
+ *                or local, e.g., file://localhost/PATH_TO/FILENAME ,
+ *           where PATH_TO is replaced with the complete path to FILENAME
+ *           using Unix shell syntax and including the device on VMS.
  *
- * note: STARTFILE must be a URL.  See the Lynx online help for more
- *       information on URLs
+ * Normally we expect you will connect to a remote site, e.g., the Lynx starting
+ * site:
  */
 #define STARTFILE "http://lynx.browser.org/"
+/*
+ * As an alternative, you may want to use a local URL.  A good choice for this
+ * is the user's home directory:
+ *#define STARTFILE "file://localhost/~/"
+ *
+ * Your choice of STARTFILE should reflect your site's needs, and be a URL that
+ * you can connect to reliably.  Otherwise users will become confused and think
+ * that they cannot run Lynx.
+ */
 
 /*****************************
  * HELPFILE must be defined as a URL and must have a
@@ -435,10 +469,10 @@
  *   for this distribution (use SHELL syntax including the device
  *   on VMS systems).
  * The default HELPFILE is:
- * http://www.crl.com/~subir/lynx/lynx_help/lynx_help_main.html
+ * http://www.trill-home.com/lynx/lynx_help/lynx_help_main.html
  *   This should be changed here or in lynx.cfg to the local path.
  */
-#define HELPFILE "http://www.crl.com/~subir/lynx/lynx_help/lynx_help_main.html"
+#define HELPFILE "http://www.trill-home.com/lynx/lynx_help/lynx_help_main.html"
 /* #define HELPFILE "file://localhost/PATH_TO/lynx_help/lynx_help_main.html" */
 
 /*****************************
@@ -478,6 +512,12 @@
  */
 #define GOTOBUFFER	  FALSE
 
+/*****************************
+ * If FTP_PASSIVE is set to TRUE here or in lynx.cfg, ftp transfers will
+ * be done in passive mode.
+ */
+#define FTP_PASSIVE	  FALSE
+
 /*****************************
  * JUMPFILE is the default local file checked for shortcut URLs when
  * the user presses the 'J' (JUMP) key.  The user will be prompted for
@@ -624,6 +664,7 @@
  *     DEC Multinational                  dec-mcs
  *     Macintosh (8 bit)                  macintosh
  *     NeXT character set                 next
+ *     HP Roman8                          hp-roman8
  *     Chinese                            euc-cn
  *     Japanese (EUC-JP)                  euc-jp
  *     Japanese (Shift_JIS)               shift_jis
@@ -653,6 +694,8 @@
  *     Hebrew (windows-1255)              windows-1255
  *     Turkish (ISO-8859-9)               iso-8859-9
  *     ISO-8859-10                        iso-8859-10
+ *     Ukrainian Cyrillic (cp866u)        cp866u
+ *     Ukrainian Cyrillic (KOI8-U)        koi8-u
  *     UNICODE (UTF-8)                    utf-8
  *     RFC 1345 w/o Intro                 mnemonic+ascii+0
  *     RFC 1345 Mnemonic                  mnemonic
@@ -666,7 +709,7 @@
  * as the preferred language.  If available, the document will be
  * transmitted in that language.  This definition can be overridden via
  * lynx.cfg.  Users also can change it via the 'o'ptions menu and save
- * that preference in their RC file. This may be a comma-separated list
+ * that preference in their RC file.  This may be a comma-separated list
  * of languages in decreasing preference.
  */
 #define PREFERRED_LANGUAGE "en"
@@ -681,7 +724,7 @@
  * values are always assumed by default.
  * If a file in that character set is available, the server will send it.
  * If no Accept-Charset header is present, the default is that any
- * character set is acceptable. If an Accept-Charset header is present,
+ * character set is acceptable.  If an Accept-Charset header is present,
  * and if the server cannot send a response which is acceptable
  * according to the Accept-Charset header, then the server SHOULD send
  * an error response with the 406 (not acceptable) status code, though
@@ -870,7 +913,7 @@
  * Showing the cursor is handy if you are a sighted user with a poor
  * terminal that can't do bold and reverse video at the same time or
  * at all.  It also can be useful to blind users, as an alternative
- * or supplement to setting LINKS_AND_FORM_FIELDS_ARE_NUMBERED or
+ * or supplement to setting LINKS_AND_FIELDS_ARE_NUMBERED or
  * LINKS_ARE_NUMBERED.
  *
  * The default defined here can be changed in lynx.cfg.  It can be
@@ -881,10 +924,11 @@
 #define SHOW_CURSOR FALSE
 
 /******************************
-* VERBOSE_IMAGES controls whether or not Lynx replaces the [LINK], [INLINE] and
-* [IMAGE] comments (for images without ALT) with filenames of these images.
-* This is extremely useful because now we can determine immediately what images
-* are just decorations (button.gif, line.gif) and what images are important.
+* VERBOSE_IMAGES controls whether or not Lynx replaces the [LINK], [INLINE]
+* and [IMAGE] comments (for images without ALT) with filenames of these
+* images.  This is extremely useful because now we can determine immediately
+* what images are just decorations (button.gif, line.gif) and what images are
+* important.
 *
 * The default defined here can be changed in lynx.cfg.
 */
@@ -914,7 +958,8 @@
  * Note that this is currently not supported with the chartrans code,
  * or rather it doesn't have an effect if translations for a display
  * character set are taken from one of the *.tbl files in src/chrtrans.
- * One would have to modify the corresponding *.tbl file for this.
+ * One would have to modify the corresponding *.tbl file to change the
+ # 7-bit replacements for these characters.
  */
 #define LY_UMLAUT
 
@@ -941,7 +986,7 @@
  * Use lynxexec for commands or scripts that generate a screen output which
  * should be held via a prompt to press <return> before returning to Lynx
  * for display of the current document.
- * Use lynxprog for programs such as mail which do require a pause before
+ * Use lynxprog for programs such as mail which do not require a pause before
  * Lynx restores the display of the current document.
  *
  * Execution scripts take the form of a standard
@@ -974,23 +1019,80 @@
  * also see src/HTInit.c to verify/change the execution
  * script extensions and/or commands.
  */
-/* #define EXEC_LINKS */
-/* #define EXEC_SCRIPTS */
+/* #define EXEC_LINKS  */
+/* #define EXEC_SCRIPTS  */
+
+#if defined(EXEC_LINKS) || defined(EXEC_SCRIPTS)
 
 /**********
+ * if ENABLE_OPTS_CHANGE_EXEC is defined, the user will be able to change
+ * the execution status within the Options Menu.
+ */
+/* #define ENABLE_OPTS_CHANGE_EXEC */
+
+/**********
+ * if NEVER_ALLOW_REMOTE_EXEC is defined,
+ * local execution of scripts or lynxexec & lynxprog URLs will be implemented
+ * only from HTML files that were accessed via a "file://localhost/" URL
+ * and the Options Menu for "Local executions links" will allow toggling
+ * only between "ALWAYS OFF" and "FOR LOCAL FILES ONLY".
+ */
+/* #define NEVER_ALLOW_REMOTE_EXEC */
+
+/*****************************
+ * These are for executable shell scripts and links.
+ * Set to FALSE unless you really know what you're
+ * doing.
+ *
+ * This only applies if you are compiling with EXEC_LINKS or
+ * EXEC_SCRIPTS defined.
+ *
+ * The first two settings:
+ * LOCAL_EXECUTION_LINKS_ALWAYS_ON
+ * LOCAL_EXECUTION_LINKS_ON_BUT_NOT_REMOTE
+ * specify the DEFAULT settings of the users execution link
+ * options (they can also be overridden in lynx.cfg), but
+ * the user may still change those options.
+ * If you do not wish the user to be able to change the
+ * execution link settings you may wish to use the command line option:
+ *    -restrictions=exec_frozen
+ *
+ * LOCAL_EXECUTION_LINKS_ALWAYS_ON will be FALSE
+ * if NEVER_ALLOW_REMOTE_EXEC has been defined.
+ *
+ * if LOCAL_EXECUTION_LINKS_ALWAYS_OFF_FOR_ANONYMOUS is true,
+ * all execution links will be disabled when the -anonymous
+ * command-line option is used.  Anonymous users are not allowed
+ * to change the execution options from within the Lynx Options Menu,
+ * so you might be able to use this option to enable execution links
+ * and set LOCAL_EXECUTION_LINKS_ON_BUT_NOT_REMOTE to TRUE
+ * to give anonymous execution-link capability without compromising
+ * your system (see comments about TRUSTED_EXEC rules in lynx.cfg ).
+ */
+
+#define LOCAL_EXECUTION_LINKS_ALWAYS_ON          FALSE
+#define LOCAL_EXECUTION_LINKS_ON_BUT_NOT_REMOTE  FALSE
+#define LOCAL_EXECUTION_LINKS_ALWAYS_OFF_FOR_ANONYMOUS FALSE
+
+#endif /*  defined(EXEC_LINKS) || defined(EXEC_SCRIPTS) */
+
+/**********
+ * *** This is for those -- e.g. DOS users -- who do not have configure;
+ * *** others should use the configure switch --enable-lynxcgi-links .
+ *
  * UNIX:
  * =====
- * CGI script support. Defining LYNXCGI_LINKS allows you to use the
+ * CGI script support.  Defining LYNXCGI_LINKS allows you to use the
  *
  *   lynxcgi:path
  *
  * URL which allows lynx to access a cgi script directly without the need for
- * a http daemon. Redirection or mime support is not supported but just about
- * everything else is. If the path is not an executable file then the URL is
- * rewritten as file://localhost and passed to the file loader. This means that
- * if your http:html files are currently set up to use relative addressing, you
- * should be able to fire up your main page with lynxcgi:path and everything
- * should work as if you were talking to the http daemon.
+ * a http daemon.  Redirection is not supported but just about everything
+ * else is.  If the path is not an executable file then the URL is
+ * rewritten as file://localhost and passed to the file loader.  This means
+ * that if your http:html files are currently set up to use relative
+ * addressing, you should be able to fire up your main page with lynxcgi:path
+ * and everything should work as if you were talking to the http daemon.
  *
  * Note that TRUSTED_LYNXCGI directives must be defined in your lynx.cfg file
  * if you wish to place restrictions on source documents and/or paths for
@@ -1010,81 +1112,41 @@
  * lynxcgi:advice  as a G)oto URL for more information and links to the
  * OSU server distribution.
  */
+#ifndef HAVE_CONFIG_H
 /* #define LYNXCGI_LINKS */
-
-#if defined(EXEC_LINKS) || defined(EXEC_SCRIPTS)
-
-/**********
- * if ALLOW_USERS_TO_CHANGE_EXEC_WITHIN_OPTIONS
- * is defined then the user will be able to change
- * the execution status within the options screen.
- */
-/* #define ALLOW_USERS_TO_CHANGE_EXEC_WITHIN_OPTIONS */
-
-/**********
- * if NEVER_ALLOW_REMOTE_EXEC is defined then local execution of
- * scripts or lynxexec and lynxprog URLs will only be implemented
- * from HTML files that were accessed via a "file://localhost/" URL,
- * and the options menu for "L)ocal executions links" will only
- * allow toggling between "ALWAYS OFF" and "FOR LOCAL FILES ONLY".
- */
-/* #define NEVER_ALLOW_REMOTE_EXEC */
-
-/*****************************
- * These are for executable shell scripts and links.
- * Set to FALSE unless you really know what you're
- * doing.
- *
- * This only applies if you are compiling with EXEC_LINKS or
- * EXEC_SCRIPTS defined.
- *
- * The first two settings:
- * LOCAL_EXECUTION_LINKS_ALWAYS_ON
- * LOCAL_EXECUTION_LINKS_ON_BUT_NOT_REMOTE
- * specify the DEFAULT setting of the users execution link
- * options, but the user may still change those options.
- * If you do not wish the user to be able to change the
- * execution link settings you may wish to use the command line option:
- *    -restrictions=exec_frozen
- *
- * LOCAL_EXECUTION_LINKS_ALWAYS_ON will be FALSE
- * if NEVER_ALLOW_REMOTE_EXEC has been defined.
- *
- * if LOCAL_EXECUTION_LINKS_ALWAYS_OFF_FOR_ANONYMOUS is
- * true all execution links will be disabled when the
- * -anonymous command line option is used.  Anonymous
- * users are not allowed to change the execution options
- * from within the Lynx options menu so you might be able
- * to use this option to enable execution links and set
- * LOCAL_EXECUTION_LINKS_ON_BUT_NOT_REMOTE to TRUE to
- * give anonymous execution link capability without compromising
- * your system (see comments about TRUSTED_EXEC rules in
- * lynx.cfg for more information).
- *
- */
-#define LOCAL_EXECUTION_LINKS_ALWAYS_ON          FALSE
-#define LOCAL_EXECUTION_LINKS_ON_BUT_NOT_REMOTE  FALSE
-#define LOCAL_EXECUTION_LINKS_ALWAYS_OFF_FOR_ANONYMOUS FALSE
-
-#endif /*  defined(EXEC_LINKS) || defined(EXEC_SCRIPTS) */
+#endif
 
 /*********************************
  *  MAIL_SYSTEM_ERROR_LOGGING will send a message to the owner of
  *  the information if there is one, every time
  *  that a document cannot be accessed!
+ *  This is just the default, it can be changed in lynx.cfg, and error
+ *  logging can be turned off with the -nolog command line option.
  *
  *  NOTE: This can generate A LOT of mail, be warned.
  */
 #define MAIL_SYSTEM_ERROR_LOGGING   FALSE  /*mail a message for every error?*/
 
+/*********************************
+ *  If a document cannot be accessed, and MAIL_SYSTEM_ERROR_LOGGING
+ *  is on and would send a message to the owner of the information,
+ *  but no owner is known, then the message will be sent to ALERTMAIL
+ *  instead - if it is defined as a non-empty email address.
+ *
+ *  NOTE: This can generate A REAL LOT of mail, be warned!!!
+ */
+/* #define ALERTMAIL "webmaster@localhost" */ /*error recipient if no owner*/
+
 /*********************************
  * If CHECKMAIL is set to TRUE, the user will be informed (via a status line
  * message) about the existence of any unread mail at startup of Lynx, and
  * will get status line messages if subsequent new mail arrives.  If a jumps
  * file with a lynxprog URL for invoking mail is available, or your html
  * pages include an mail launch file URL, the user thereby can access mail
- * and read the messages.  The checks and status line reports will not be
- * performed if Lynx has been invoked with the -restrictions=mail switch.
+ * and read the messages.
+ * This is just the default, it can be changed in lynx.cfg.  The checks and
+ * status line reports will not be performed if Lynx has been invoked with
+ * the -restrictions=mail switch.
  *
  *  VMS USERS !!!
  * New mail is normally broadcast as it arrives, via "unsolicited screen
@@ -1096,30 +1158,28 @@
 #define CHECKMAIL	FALSE	/* report unread and new mail messages */
 
 /*********************************
- * VI_KEYS can be turned on by the user in the options
- * screen or the .lynxrc file.  This is just the default.
+ * Vi or Emacs movement keys.  These are defaults,
+ * which can be changed in lynx.cfg , the Options Menu or .lynxrc .
  */
-#define VI_KEYS_ALWAYS_ON	FALSE /* familiar h,j,k, & l */
+#define VI_KEYS_ALWAYS_ON	FALSE /* familiar h j k l */
+#define EMACS_KEYS_ALWAYS_ON	FALSE /* familiar ^N ^P ^F ^B */
 
 /*********************************
- * EMACS_KEYS can be turned on by the user in the options
- * screen or the .lynxrc file.  This is just the default.
- */
-#define EMACS_KEYS_ALWAYS_ON	FALSE /* familiar ^N, ^P, ^F, ^B */
-
-/*********************************
- * DEFAULT_KEYPAD_MODE specifies whether by default the user
- * has numbers that work like arrows or else numbered links
- * DEFAULT KEYPAD MODE may be set to
- *	NUMBERS_AS_ARROWS   or
- *	LINKS_ARE_NUMBERED  or
- *	LINKS_AND_FORM_FIELDS_ARE_NUMBERED
+ * DEFAULT_KEYPAD_MODE may be set to NUMBERS_AS_ARROWS
+ *                                or LINKS_ARE_NUMBERED
+ *                                or LINKS_AND_FIELDS_ARE_NUMBERED
+ * to specify whether numbers (e.g. [10]) appear before all links,
+ * allowing immediate access by entering the number on the keyboard,
+ * or numbers on the numeric key-pad work like arrows;
+ * the 3rd option causes form fields also to be preceded by numbers.
+ * The first two options (but not the last) can be changed in lynx.cfg
+ * and all three can be changed via the Options Menu.
  */
 #define DEFAULT_KEYPAD_MODE	NUMBERS_AS_ARROWS
 
 /********************************
  * The default search.
- * This is a default that can be overridden by the user!
+ * This is a default that can be overridden in lynx.cfg or by the user!
  */
 #define CASE_SENSITIVE_ALWAYS_ON    FALSE /* case sensitive user search */
 
@@ -1132,7 +1192,7 @@
  * to be treated as TRUE via -restrictions=dotfiles (or -anonymous, which sets
  * this and most other restrictions).
  *
- * If it's FALSE at startup of Lynx, the user can regulate it via the
+ * If it is FALSE at startup of Lynx, the user can regulate it via the
  * 'o'ptions menu, and may save the preference in the RC file.
  */
 #define NO_DOT_FILES    TRUE  /* disallow access to dot files */
@@ -1186,6 +1246,55 @@
  */
 #define QUIT_DEFAULT_YES	TRUE
 
+/********************************
+ * If TEXT_SUBMIT_CONFIRM_WANTED is defined (to anything), the user will be
+ * prompted for confirmation before Lynx submits a form with only one input
+ * field (of type text) to the server, after the user has pressed <return>
+ * or <enter> on the field.  Since the is no other way such as a "submit"
+ * button to submit, normally the form gets submitted automatically in this
+ * case, but some users may find this surprising and expect <return> to just
+ * move to the next link as for other text entry fields.
+ */
+/* #define TEXT_SUBMIT_CONFIRM_WANTED */
+
+/********************************
+ * If TEXTFIELDS_MAY_NEED_ACTIVATION is defined (to anything),
+ * the option TEXTFIELDS_NEED_ACTIVATION in lynx.cfg or the command
+ * line option -tna can be used to require explicit activation
+ * before text input fields can be changed with the built-in line
+ * editor.
+ */
+
+#define TEXTFIELDS_MAY_NEED_ACTIVATION
+
+/********************************
+ * The following three definitions control some aspects of extended
+ * textarea handling.  TEXTAREA_EXPAND_SIZE is the number of new empty
+ * lines that get appended at the end of a textarea by a GROWTEXTAREA
+ * key.  If TEXTAREA_AUTOGROW is defined (to anything), <return> or
+ * <enter> in the last line of a textarea automatically extends the
+ * area by adding a new line.  If TEXTAREA_AUTOEXTEDIT is defined (to
+ * anything), a key mapped to DWIMEDIT will invoke the external editor
+ * like EDITTEXTAREA when used in a text input field.  Comment those
+ * last two definitions out to disable the corresponding behavior.
+ * See under KEYMAP in lynx.cfg for mapping keys to GROWTEXTAREA or
+ * DWIMEDIT actions.
+ */
+#define TEXTAREA_EXPAND_SIZE  5
+#define TEXTAREA_AUTOGROW
+#define TEXTAREA_AUTOEXTEDIT
+
+/********************************
+ * If BUILTIN_SUFFIX_MAPS is defined (to anything), default mappings
+ * for file extensions (aka suffixes) will be compiled in (see
+ * src/HTInit.c).  By removing the definition, the default mappings
+ * are suppressed except for a few very basic ones for text/html.
+ * See GLOBAL_EXTENSION_MAP, PERSONAL_EXTENSION_MAP above and SUFFIX,
+ * SUFFIX_ORDER in lynx.cfg for other ways to map file extensions.
+ */
+
+#define BUILTIN_SUFFIX_MAPS
+
 /********************************
  * These definitions specify files created or used in conjunction
  * with traversals.  See CRAWL.ANNOUNCE for more information.
@@ -1201,9 +1310,23 @@
  * program execution.  See the comments in LYMessages_en.h for
  * information on translating or customizing them for your site.
  */
-#ifndef HTTELNET_H
-#include "LYMessages_en.h"
-#endif /* !HTTELNET_H */
+#ifndef LYMESSAGES_EN_H
+#include <LYMessages_en.h>
+#endif /* !LYMESSAGES_EN_H */
+
+
+/****************************************************************
+ * DEFAULT_VISITED_LINKS may be set to one or more of
+ *					VISITED_LINKS_AS_FIRST_V
+ *					VISITED_LINKS_AS_TREE
+ *					VISITED_LINKS_AS_LATEST
+ *					VISITED_LINKS_REVERSE
+ * to change the organization of the Visited Links page.
+ *
+ * (Not all combinations are meaningful; see src/LYrcFile.c for a list
+ * in the visited_links_tbl table).
+ */
+#define DEFAULT_VISITED_LINKS (VISITED_LINKS_AS_LATEST | VISITED_LINKS_REVERSE)
 
 
 /****************************************************************
@@ -1213,28 +1336,18 @@
 
 #define LYNX_NAME "Lynx"
 /* The strange-looking comments on the next line tell PRCS to replace
- * the version definition with the Project Version on checkout. Just
+ * the version definition with the Project Version on checkout.  Just
  * ignore it. - kw */
 /* $Format: "#define LYNX_VERSION \"$ProjectVersion$\""$ */
-#define LYNX_VERSION "2.8.1rel.2"
+#define LYNX_VERSION "2.8.3rel.1"
 #define LYNX_WWW_HOME "http://lynx.browser.org/"
-#define LYNX_WWW_DIST "http://www.slcc.edu/lynx/current/"
-#define LYNX_RELEASE TRUE
+#define LYNX_WWW_DIST "http://lynx.isc.org/current/"
 /* $Format: "#define LYNX_DATE \"$ProjectDate$\""$ */
-#define LYNX_DATE "Tue, 27 Oct 1998 19:08:12 -0700"
+#define LYNX_DATE "Sun, 23 Apr 2000 18:44:03 -0700"
 #define LYNX_DATE_OFF 5		/* truncate the automatically-generated date */
 #define LYNX_DATE_LEN 11	/* truncate the automatically-generated date */
-#define LYNX_RELEASE_DATE "1998"
 
-#ifndef MAXINT
-#define MAXINT 2147483647	/* max integer */
-#endif /* !MAXINT */
-#define MAXBASE 100		/* max length of base directory */
-#define MAXHIGHLIGHT 160	/* max length of highlighted text */
-#define MAXTARGET 130		/* max length of target string */
 #define LINESIZE 1024		/* max length of line to read from file */
-#define MAXFNAME 1280		/* max filename length DDD/FILENAME.EXT */
-#define MAXCOMMAND MAXFNAME	/* max length of command should be the same */
 #define MAXHIST  1024		/* max links we remember in history */
 #define MAXLINKS 1024		/* max links on one screen */
 
@@ -1243,7 +1356,9 @@
 #endif
 
 #define MAXCHARSETS 60		/* max character sets supported */
-#define MAXCHARSETSP 61		/* always one more than MAXCHARSETS */
+#define TRST_MAXROWSPAN 10000	/* max rowspan accepted by TRST code */
+#define TRST_MAXCOLSPAN 1000	/* max colspan and COL/COLGROUP span accepted */
+#define SAVE_TIME_NOT_SPACE	/* minimize number of some malloc calls */
 
 /* Win32 may support more, but old win16 helper apps may not. */
 #if defined(__DJGPP__) || defined(_WINDOWS)
@@ -1260,20 +1375,47 @@
 /*
 **  Use the VMS port of gzip for uncompressing both .Z and .gz files.
 */
-#define UNCOMPRESS_PATH  "gzip -d"
-#define GZIP_PATH "gzip"
+#define UNCOMPRESS_PATH "gzip -d"
+#define COPY_PATH	"copy/nolog/noconf"
+#define GZIP_PATH       "gzip"
+#define BZIP2_PATH      "bzip2"
+#define TELNET_PATH     "telnet"
+#define TN3270_PATH     "tn3270"
+#define RLOGIN_PATH     "rlogin"
 
 #else
 
 #ifdef DOSPATH
-/*  Something has to be defined for this or we don't compile. */
+
+#ifdef _WINDOWS
+#ifdef USE_ALT_BLAT_MAILER
+#define SYSTEM_MAIL		"BLAT"
+#define SYSTEM_MAIL_FLAGS	""
+#else
+#define SYSTEM_MAIL		"BLATJ"
+#define SYSTEM_MAIL_FLAGS	""
+#endif
+#else
+/* have to define something... */
 #define SYSTEM_MAIL "sendmail"
 #define SYSTEM_MAIL_FLAGS "-t -oi"
+#endif
+
 /*
-**  Following executables may be sought from your PATH at run-time.
-**  To get those programs look for GNU-port stuff elsewhere. Currently,
-**  if compiled with -DUSE_ZLIB and without -DDIRED_SUPPORT (default),
-**  you need only "cp.exe" from the list below.
+**  The following executables may be used at run time.  Unless you change
+**  the definitions to include the full directories, they will be sought
+**  from your PATH at run-time; they should be available as "cp.exe",
+**  "mv.exe" and so on.  To get those programs look for GNU-port stuff
+**  elsewhere.
+**  Currently, if compiled with -DUSE_ZLIB and without -DDIRED_SUPPORT
+**  (default), the following from the list below are required:
+**  MV_PATH   (mv.exe) - for bookmark handling (DEL_BOOKMARK command)
+**  UNCOMPRESS_PATH    - for automatic decompression of files in Unix
+**                       compress format
+**  TELNET_PATH, TN3270_PATH, RLOGIN_PATH - for access to "telnet:",
+**                                         "tn3270:", and "rlogin:" URLs.
+**  If they are not defined right, the corresponding operations may fail
+**  in unexpected and obscure ways!
 **
 **    WINDOWS/DOS
 **    ===========
@@ -1283,6 +1425,7 @@
 #define UUDECODE_PATH   "uudecode"
 #define ZCAT_PATH       "zcat"
 #define GZIP_PATH       "gzip"
+#define BZIP2_PATH      "bzip2"
 #define INSTALL_PATH    "install"
 #define TAR_PATH        "tar"
 #define TOUCH_PATH      "touch"
@@ -1291,8 +1434,12 @@
 #define MKDIR_PATH      "mkdir"
 #define MV_PATH         "mv"
 #define RM_PATH         "rm"
-#define COPY_PATH       "cp"
+/* COPY_PATH is not required for DOSPATH any more (implemented directly) */
+/* #define COPY_PATH       "cp" */
 #define CHMOD_PATH      "chmod"
+#define TELNET_PATH     "telnet"
+#define TN3270_PATH     "tn3270"
+#define RLOGIN_PATH     "rlogin"
 
 #else	/* Unix */
 	/* Standard locations are defined via the configure script.  When
@@ -1304,6 +1451,45 @@
 #endif /* VMS */
 
 
+/***************************** 
+ * SUPPORT_MULTIBYTE_EDIT provides better support of CJK characters to
+ * Lynx's Line Editor.  JIS X0201 Kana is partially supported.  The
+ * reason why I didn't support it fully is I think supporting it is not
+ * required so much and I don't have an environment to test it. - TH
+ */
+#define SUPPORT_MULTIBYTE_EDIT
+
+/*****************************
+ * USE_TH_JP_AUTO_DETECT, CONV_JISX0201KANA_JISX0208KANA,  
+ * and KANJI_CODE_OVERRIDE are the macros for Japanese. - TH 
+ */ 
+/***************************** 
+ * USE_TH_JP_AUTO_DETECT enables a new Japanese charset detection routine. 
+ * With the old detection strategy, Lynx always thought a document was 
+ * written in mixture of three kanji codes (JIS, EUC and SJIS).  The new 
+ * strategy is for Lynx to first assume the document is written in one code 
+ * or JIS + one other kanji code (JIS, EUC, SJIS, EUC+JIS and SJIS+JIS). 
+ * The first assumption is usually correct, but if the assumption is wrong, 
+ * Lynx falls back to the old assumption of the three kanji codes mixed. 
+ */ 
+#define USE_TH_JP_AUTO_DETECT 
+ 
+/***************************** 
+ * If CONV_JISX0201KANA_JISX0208KANA is set, Lynx will convert 
+ * JIS X0201 Kana to JIS X0208 Kana, i.e., convert half-width kana 
+ * to full-width. 
+ */ 
+#define CONV_JISX0201KANA_JISX0208KANA 
+ 
+/***************************** 
+ * Uncomment the following line to enable the kanji code override routine. 
+ * The code can be changed by pressing ^L.  More precisely, this allows 
+ * the user to override the assumption about the kanji code for the document 
+ * which Lynx has made on the basis of a META tag and HTTP response. 
+ */ 
+/*#define KANJI_CODE_OVERRIDE */ 
+ 
+
 /****************************************************************
  *  Section 4.  Things you MUST check only if you plan to use Lynx
  *              in an anonymous account (allow public access to Lynx).
@@ -1357,7 +1543,10 @@
 
 /*******************************
  * set to FALSE if you don't want users of your anonymous
- * account to be able to read news
+ * account to be able to read news OR post news articles.
+ * These flags apply to "news", "nntp", "newspost", and "newsreply"
+ * URLs, but not to "snews", "snewspost", or "snewsreply"
+ * in case they are supported.
  */
 #define CAN_ANONYMOUS_INSIDE_DOMAIN_READ_NEWS	TRUE
 #define CAN_ANONYMOUS_OUTSIDE_DOMAIN_READ_NEWS	FALSE
@@ -1417,6 +1606,48 @@
  */
 #define CAN_ANONYMOUS_PRINT	FALSE
 
+/*******************************
+ * set to FALSE if users with anonymous restrictions should
+ * not be able to view configuration file (lynx.cfg) info
+ * via special LYNXCFG: links.  (This does not control access
+ * to lynx.cfg as a normal file, e.g., through a "file:" URL,
+ * if other restrictions allow that.)
+ */
+#define CAN_ANONYMOUS_VIEW_LYNXCFG_INFO			FALSE
+
+/*******************************
+ * set to FALSE if users with anonymous restrictions should
+ * not be able to view extended configuration file (lynx.cfg)
+ * info @@@ or perform special config info functions (reloading
+ * at run-time) via special LYNXCFG: links @@@.  This only applies
+ * if the lynxcfg_info" restriction controlled by the previous
+ * item is not in effect and if Lynx has been compiled without
+ * NO_CONFIG_INFO defined (--disable-config-info wasn't used
+ * if Lynx was built with the autoconf configure script).
+ * The extended info may include details on configuration file
+ * names and location and links for reading the files, as well
+ * as information on nesting of included configuration files.
+ */
+#define CAN_ANONYMOUS_VIEW_LYNXCFG_EXTENDED_INFO	FALSE
+
+/*******************************
+ * set to FALSE if users with anonymous restrictions should
+ * not be able to view information on compile time configuration
+ * via special LYNXCOMPILEOPTS: links.  This only applies
+ * if the autoconf configure script was used to build Lynx
+ * AND --disable-config-info wasn't used, otherwise this
+ * special URL scheme isn't recognized anyway.
+ */
+#define CAN_ANONYMOUS_VIEW_COMPILEOPTS_INFO		FALSE
+
+/*******************************
+ * set to FALSE if you don't want users of your anonymous
+ * account to be able to 'g'oto special URLs for showing
+ * configuration info (LYNXCFG: and LYNXCOMPILEOPTS:) if
+ * they are otherwise allowed.
+ */
+#define CAN_ANONYMOUS_GOTO_CONFIGINFO		FALSE
+
 /*****************************
  * Be sure you have read about and set defines above in Sections
  * 1, 2 and 3 that could  affect Lynx in an anonymous account,
